@@ -370,7 +370,7 @@ checkINFOdigests() {
 			;;
 	esac
 	tmpfile="${tmpdir}/checkINFOdigests"
-	"${CAT}" catalog/INFO | INFO_to_digests "$dig_type" | "${SORT}"  | tee /tmp/unscreme_$dig_type >"$tmpfile"
+	"${CAT}" catalog/INFO | INFO_to_digests "$dig_type" | "${SORT}"  >"$tmpfile"
 	grep ^. $tmpfile 1>/dev/null
 	case "$?" in
 		0) ;;
@@ -563,7 +563,7 @@ check_files2() {
 	case $? in 0) ;; *) echo "error mkfifo" 1>&2; exit 1 ;; esac
 
 	(${CAT} "${package_dir}"/catalog/dfiles/files | ${SED} -e 's@/$@@' | ${SORT} | dd of="$tmpfifo" 2>/dev/null) &
-	${TAR} cvf /dev/null "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${DIFF} -h - "$tmpfifo"
+	find "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${DIFF} -h - "$tmpfifo"
 	ret_f=$?
 	wait
 	"${RM}" -f "$tmpfifo"
@@ -573,7 +573,7 @@ check_files2() {
 
 check_files_by_md5() {
 	csum=`${CAT} "${package_dir}"/catalog/dfiles/files | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
-	fsum=`${TAR} cvf /dev/null "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
+	fsum=`find "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
 	# OLD slow way:  fsum=`${TAR} cf - ${package_dir} | ${TAR} tf - | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
 	case "$csum" in
 		"") return 1
@@ -653,7 +653,7 @@ check_digest_from_catalog_files() {
 			cat
 		fi
 		) | sed -e 's@/$@@' |
-	"${GREP}" -v "${package_dir}"/catalog
+	"${GREP}" -v ^"${package_dir}"/catalog
 	) | \
 		create_archive_stream "$tarcreateutil" | ${digest_program} | sed -e 's/ .*$//'`
 
@@ -967,8 +967,17 @@ checkdigest|checksig)
 		
 		if [ -f "${sha512sumfile}" ]; then
 			printf "${premg}Checking Archive sha512 "
-			check_digest_from_catalog_files "$SHA512SUM" "${sha512sumfile}" "" "$tarcreateutil"
-			sha512sum_ret=$?
+			case "$SHA512SUM" in
+				/)
+					echo \
+					"${premg}The sha512sum utility was not found" 1>&2
+					sha512sum_ret=0
+					;;
+				*)
+					check_digest_from_catalog_files "$SHA512SUM" "${sha512sumfile}" "" "$tarcreateutil"
+					sha512sum_ret=$?
+					;;
+			esac
 		else
 			sha512sum_ret=99
 		fi
