@@ -124,6 +124,11 @@ typedef struct {
 	int g_do_cleanshM;
 	char g_gpufieldM[30];
 	int g_gpuflagM;
+	int g_do_createM; /* used by swcopy */
+	int g_do_extractM; /* used by swcopy */
+	STRAR * g_sourcepath_listM; /* used by swcopy */
+	int g_no_extractM; /* used by swcopy */
+	int g_do_show_linesM; /* used by swverify */
 } GB;
 
 #define G_opta (G->optaM)
