@@ -56,7 +56,7 @@
 #define PROGNAME                "swverify"
 #define SW_UTILNAME             PROGNAME
 #define SWPROG_VERSION          SWVERIFY_VERSION
-#define WOPT_LAST		214   /* Highest option number */
+#define WOPT_LAST		215   /* Highest option number */
 
 static char progName[] = PROGNAME;
 static char * CHARTRUE = "True";
@@ -148,7 +148,7 @@ fprintf(fp, "%s",
 "      --no-getconf      Don't use the getconf utility to set the PATH variable\n"
 "                        on the target host.\n"
 "      --use-getconf     This causes getconf to be used.\n"
-"      --shell-command={posix|sh|bash|ksh|dash}  bash is the default\n"
+"      --shell-command={detect|posix|sh|bash|ksh}  detect is the default\n"
 "      --ssh-options=options      single letter ssh options. e.g. --ssh-options=1\n"
 "   -A,--A,--enable-ssh-agent-forwarding\n"
 "      --a,--disable-ssh-agent-forwarding\n"
@@ -169,6 +169,7 @@ fprintf(fp, "%s",
 "      --check-mtime=true|false\n"
 "      --check-scripts=true|false    Not implemented\n"
 "      --check-volatile=true|false\n"
+"      --show-info     show the data that is being compared\n"
 "\n"
 " Debugging Controls:\n"
 "\n"
@@ -210,7 +211,7 @@ fprintf(fp, "%s",
 "               ~/.swbis/swbisdefaults\n"
 "        Command Line Syntax : -x option=option_argument\n"
 "  swverify.swbis_no_getconf		 = true # getconf is used\n"
-"  swverify.swbis_shell_command	         = bash\n"
+"  swverify.swbis_shell_command	         = detect\n"
 "  swverify.swbis_no_remote_kill	 = false\n"
 "  swverify.swbis_any_format             = false\n"
 "  swverify.swbis_forward_agent          = true\n"
@@ -224,38 +225,6 @@ fprintf(fp, "%s",
 	return 0;
 }
 
-void
-process_pax_command(GB * G, char * command, char * type, int sw_e_num)
-{
-	char * xcmd;
-
-	if (strcmp(type, "write") == 0) {
-		xcmd = swc_get_pax_write_command(
-			G->g_pax_write_commands,
-			command,
-			G->g_verboseG, (char*)NULL);
-	} else if (strcmp(type, "read") == 0) {
-		xcmd = swc_get_pax_read_command(
-			G->g_pax_read_commands,
-			command,
-			0, 0, (char*)NULL);
-	} else if (strcmp(type, "remove") == 0) {
-		xcmd = swc_get_pax_remove_command(
-			G->g_pax_remove_commands,
-			command, 0, (char*)NULL);
-	} else {
-		xcmd = NULL;
-		sw_e_msg(G, "internal error\n");
-		LCEXIT(1);
-	}
-	if (xcmd == NULL) {
-		sw_e_msg(G, "illegal pax %s command: %s \n",
-			type, command);
-			LCEXIT(1);
-	}
-	set_opta(G->optaM, sw_e_num, command);
-}
-
 static
 void
 abort_script(SWICOL * swicol, GB * G, int fd)
@@ -643,6 +612,7 @@ main(int argc, char **argv)
 		{"check-contents", 1, 0, 212},
 		{"check-permissions", 1, 0, 213},
 		{"check-scripts", 1, 0, 214},
+		{"show-info", 0, 0, 215},
 		{0, 0, 0, 0}
              };
 
@@ -879,9 +849,9 @@ main(int argc, char **argv)
 	eopt_verbose			= "1";
 	wopt_no_getconf			= CHARTRUE;
 	wopt_no_remote_kill		= CHARFALSE;
-	wopt_shell_command		= "bash";
-	wopt_local_pax_write_command 	= "tar"; 
-	wopt_remote_pax_write_command 	= "tar";
+	wopt_shell_command		= "detect";
+	wopt_local_pax_write_command 	= "detect"; 
+	wopt_remote_pax_write_command 	= "detect";
 	wopt_local_pax_read_command 	= "tar";
 	wopt_remote_pax_read_command 	= "tar";
 	wopt_local_pax_remove_command 	= "tar";
@@ -1212,39 +1182,39 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			break;
 		case 175:
 			wopt_local_pax_write_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_write_command, "write",
+			swc_process_pax_command(G, wopt_local_pax_write_command, "write",
 						SW_E_swbis_local_pax_write_command);
 			break;
 		case 176:
 			wopt_remote_pax_write_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_write_command, "write",
+			swc_process_pax_command(G, wopt_remote_pax_write_command, "write",
 						SW_E_swbis_remote_pax_write_command);
 			break;
 		case 177:
 			wopt_local_pax_write_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_write_command, "write",
+			swc_process_pax_command(G, wopt_local_pax_write_command, "write",
 						SW_E_swbis_local_pax_write_command);
 
 			wopt_remote_pax_write_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_write_command, "write",
+			swc_process_pax_command(G, wopt_remote_pax_write_command, "write",
 						SW_E_swbis_remote_pax_write_command);
 			break;
 		case 178:
 			wopt_remote_pax_read_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_read_command, "read",
+			swc_process_pax_command(G, wopt_remote_pax_read_command, "read",
 						SW_E_swbis_remote_pax_read_command);
 			break;
 		case 179:
 			wopt_local_pax_read_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_read_command, "read",
+			swc_process_pax_command(G, wopt_local_pax_read_command, "read",
 						SW_E_swbis_local_pax_read_command);
 			break;
 		case 180:
 			wopt_local_pax_read_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_read_command, "read",
+			swc_process_pax_command(G, wopt_local_pax_read_command, "read",
 						SW_E_swbis_local_pax_read_command);
 			wopt_remote_pax_read_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_read_command, "read",
+			swc_process_pax_command(G, wopt_remote_pax_read_command, "read",
 						SW_E_swbis_remote_pax_read_command);
 			break;
 		case 181:
@@ -1252,22 +1222,22 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			break;
 		case 182:
 			wopt_local_pax_write_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_write_command, "write",
+			swc_process_pax_command(G, wopt_local_pax_write_command, "write",
 						SW_E_swbis_local_pax_write_command);
 			wopt_remote_pax_write_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_write_command, "write",
+			swc_process_pax_command(G, wopt_remote_pax_write_command, "write",
 						SW_E_swbis_remote_pax_write_command);
 			wopt_local_pax_read_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_read_command, "read",
+			swc_process_pax_command(G, wopt_local_pax_read_command, "read",
 						SW_E_swbis_local_pax_read_command);
 			wopt_remote_pax_read_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_read_command, "read",
+			swc_process_pax_command(G, wopt_remote_pax_read_command, "read",
 						SW_E_swbis_remote_pax_read_command);
 			wopt_local_pax_remove_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_remove_command, "remove",
+			swc_process_pax_command(G, wopt_local_pax_remove_command, "remove",
 						SW_E_swbis_local_pax_remove_command);
 			wopt_remote_pax_remove_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_remove_command, "remove",
+			swc_process_pax_command(G, wopt_remote_pax_remove_command, "remove",
 						SW_E_swbis_remote_pax_remove_command);
 			break;
 		case 184:
@@ -1284,18 +1254,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			break;
 		case 188:
 			wopt_shell_command = strdup(optarg);
-			if (
-				strcmp(wopt_shell_command, "bash") &&
-				strcmp(wopt_shell_command, "sh") &&
-				strcmp(wopt_shell_command, "ksh") &&
-				strcmp(wopt_shell_command, "ash") &&
-				strcmp(wopt_shell_command, "dash") &&
-				strcmp(wopt_shell_command, "posix")
-			) {
-				sw_e_msg(G, "illegal shell command: %s \n",
-					wopt_shell_command);
-				exit(1);
-			}
+			swc_check_shell_command_key(G, wopt_shell_command);
 			set_opta(opta, SW_E_swbis_shell_command, wopt_shell_command);
 			break;
 		case 195:
@@ -1332,20 +1291,20 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			break;
 		case 205:
 			wopt_remote_pax_remove_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_remove_command, "remove",
+			swc_process_pax_command(G, wopt_remote_pax_remove_command, "remove",
 						SW_E_swbis_remote_pax_remove_command);
 			break;
 		case 206:
 			wopt_local_pax_remove_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_remove_command, "remove",
+			swc_process_pax_command(G, wopt_local_pax_remove_command, "remove",
 						SW_E_swbis_local_pax_remove_command);
 			break;
 		case 207:
 			wopt_local_pax_remove_command = strdup(optarg);
-			process_pax_command(G, wopt_local_pax_remove_command, "remove",
+			swc_process_pax_command(G, wopt_local_pax_remove_command, "remove",
 						SW_E_swbis_local_pax_remove_command);
 			wopt_remote_pax_remove_command = strdup(optarg);
-			process_pax_command(G, wopt_remote_pax_remove_command, "remove",
+			swc_process_pax_command(G, wopt_remote_pax_remove_command, "remove",
 						SW_E_swbis_remote_pax_remove_command);
 			break;
 		case 208:
@@ -1381,6 +1340,9 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			SWLIB_ASSERT(optionEnum > 0);
 			set_opta(opta, optionEnum, optarg);
 			break;
+		case 215:
+			G->g_do_show_linesM = 1;
+			break;
 		default:
 			sw_e_msg(G, "error processing implementation extension option : %s\n",
 				cplob_get_list(w_arglist)[w_option_index]);
@@ -1505,6 +1467,9 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 	
 	use_no_getconf = swc_is_option_true(wopt_no_getconf);	
 
+	/* Validate the pax command */
+	swc_check_all_pax_commands(G);
+
 	/* Configure the standard I/O usages.  */
 
 	fver = G->g_vstderr;
