@@ -45,6 +45,7 @@
 #include "atomicio.h"
 #include "swutilname.h"
 #include "globalblob.h"
+#include "swproglib.h"
 #include "swi.h"
 
 char * CHARTRUE = "true";
@@ -56,8 +57,10 @@ static
 ssize_t
 try_nibble(int fd, void * buf, int timeout)
 {
+
 	return read(fd, buf, 1); /* Must block */
-	/* Dead code
+
+	/* ++++ Dead code ++++
 	int flags;
 	int ret;
 	int readret;
@@ -362,8 +365,8 @@ write_error_line(GB * G, struct sw_logspec * logspec, int vofd, int vefd, STROB
 			 do_log = 1;
 		}
 		if (verbose_level > 0 ) {
-			E_DEBUG("");
 			int is_ssh = 0;
+			E_DEBUG("");
 			if (
 				is_unix_sys_error(sptr) || 
 				(is_ssh=is_ssh_sys_error(sptr)) || 
@@ -981,7 +984,12 @@ swc_run_ssh_command (GB * G,
 	if (source_file_size) *source_file_size = 0;
 	
 	if (is_local_fd) *is_local_fd = 0;
-	if (is_no_fork_ok > 0 && nhops == 0 && *path == '/') {
+	if (
+		is_no_fork_ok > 0 &&
+		nhops == 0 &&
+		*path == '/' &&
+		1
+	) {
 		/*
 		* Optimization for Local source file.
 		*/
@@ -999,7 +1007,9 @@ swc_run_ssh_command (GB * G,
 			return -2;
 		}	
 		if (
-			S_ISREG(st.st_mode)
+			S_ISREG(st.st_mode) &&
+			G->g_do_createM == 0 &&
+			1	
 		) {
 
 			if (source_file_size) {
@@ -1623,7 +1633,8 @@ swc_get_pax_write_command(struct g_pax_write_command g_pax_write_commands[],
 	writecmd = &(g_pax_write_commands[i]);
 	id = writecmd->idM;
 	while (id) {
-		if (strcmp(keyid, id) == 0) return writecmd->commandM;
+		if (strcmp(keyid, id) == 0)
+			return writecmd->commandM;
 		i++;
 		writecmd = &(g_pax_write_commands[i]);
 		id = writecmd->idM;
@@ -1708,6 +1719,43 @@ swc_set_boolean_x_option(struct extendedOptions * opta,
 	*/
 	set_opta(opta, nopt, arg);
 }
+		
+static
+STROB *
+make_multiplepathlist(STRAR * sourcepath_listM)
+{
+	char * s;
+	char * dirname;
+	char * dir2;
+	char * base;
+	int n;
+	STROB * tmp;
+	STROB * strb;
+
+	strb = strob_open(24);
+	tmp = strob_open(24);
+	dirname = NULL;
+	n = 0;
+
+	while ((s=strar_get(sourcepath_listM, n))) {
+		dir2 = swlib_dirname(tmp, s);
+		if (dirname == NULL) {
+			dirname = strdup(dir2);
+		}
+		if (strcmp(dir2, dirname)) {
+			/* error, not the same relative root,
+			   this condition is not supported */
+			strob_close(strb);
+			return NULL;
+		}
+		base = swlib_basename(tmp, s);
+		strob_sprintf(strb, 1, "\"%s\" ", base);	
+		n++;
+	}	
+	strob_close(tmp);
+	if (dirname) free(dirname);
+	return strb;
+}
 
 int
 swc_write_source_copy_script(GB * G,
@@ -1722,25 +1770,32 @@ swc_write_source_copy_script(GB * G,
 	char * blocksize
 	)
 {
+	int n;
 	int ret;
+	int n_source_specs;
 	char * dirname, *path;
 	char * pax_write_command;
+	char * xx;
 	STROB * buffer, *opt1, *opt2;
+	STROB * buffer_new;
 	STROB * subsh, *subsh2;
 	STROB * tmp;
 	STROB * tmp1;
 	STROB * tmp2;
+	STROB * multiplepathlist;
 	STROB * set_vx;
 	STROB * to_devnull;
 	int vlv;
 
 	vlv = G->g_verboseG;
+
 	/*
 	 * error for source script is >0
 	 */
 	if (strstr(sourcepath, "..") == sourcepath) { return 1; }
 	if (strstr(sourcepath, "../")) { return 1; }
 	if (swlib_is_sh_tainted_string(sourcepath)) { return 1; }
+
 	/*
 	 * Squash the trailing slash.
 	 */
@@ -1752,6 +1807,7 @@ swc_write_source_copy_script(GB * G,
 	}
 
 	buffer = strob_open(1);
+	buffer_new = strob_open(1);
 	to_devnull = strob_open(10);
 	opt1 = strob_open(1);
 	opt2 = strob_open(1);
@@ -1813,13 +1869,44 @@ swc_write_source_copy_script(GB * G,
 		path = strdup(".");	
 	}
 
+	if (G->g_sourcepath_listM)
+		n_source_specs = strar_num_elements(G->g_sourcepath_listM);
+	else
+		n_source_specs = 1;
+
+	/* fprintf(stderr, "JL n_source_specs = %d\n", n_source_specs); */
+	/* fprintf(stderr, "JL do_create = %d\n", G->g_do_createM); */
+
+	if (n_source_specs > 1)
+		multiplepathlist = make_multiplepathlist(G->g_sourcepath_listM);
+	else
+		multiplepathlist = NULL;
+
+	if (n_source_specs > 1) {
+		if (multiplepathlist == NULL)
+			n_source_specs = 1;
+	}
+
+	/* if (multiplepathlist) fprintf(stderr, "JL MM=%s\n", strob_str(multiplepathlist)); */
+	
 	strob_sprintf(tmp1, 0, "%s", path),
 	strob_sprintf(tmp2, 0, "%s", sourcepath),
-	ret = swlib_writef(ofd, buffer, 	
+
+	strob_strcpy(buffer_new, "");
+
+	if (strcmp(get_opta(G->optaM, SW_E_swbis_shell_command), SWC_SHELL_KEY_DETECT) == 0) {
+		swpl_bashin_detect(buffer_new);
+	}
+
+	/* ret = swlib_writef(ofd, buffer,  */
+	strob_sprintf(buffer_new, STROB_DO_APPEND,
 		"echo " SWBIS_TARGET_CTL_MSG_125 ": " KILL_PID "\n"
+		CSHID
 		"%s\n"
 		"%s"
 		"%s"
+		"do_create=\"%s\"\n"
+		"multiple_source=\"%s\"\n"
 		"blocksize=\"%s\"\n"
 		"dirname=\"%s\"\n"
 		"path=\"%s\"\n"
@@ -1827,17 +1914,26 @@ swc_write_source_copy_script(GB * G,
 		"sw_retval=0\n"
 		"sb__delaytime=%d\n"
 
+		CSHID
 		"if test -d \"$dirname\"; then\n"
 		"	cd \"$dirname\"\n"
 		"	sw_retval=$?\n"
 		"	case $sw_retval in\n"
 		"	0)\n"
-		"	if test -d \"$path\" -a -r \"$path\"; then\n"
-				"\t\t%s\n"
-				"\t\t%s\n"
-				"\t\t%s\n"
-		"		%s \"$path\"\n"
-		"		sw_retval=$?\n"
+		"	if test \\( -d \"$path\" -a -r \"$path\" \\) -o \\( \"$do_create\" -a -r \"$path\" \\); then\n"
+		"		\t\t%s\n"
+		"		\t\t%s\n"
+		"		\t\t%s\n"
+		"		case \"$multiple_source\" in\n"
+		"			\"\")\n"
+		"			%s \"$path\"\n"
+		"			sw_retval=$?\n"
+		"			;;\n"
+		"			x)\n"
+		"			%s %s\n"
+		"			sw_retval=$?\n"
+		"			;;\n"
+		"		esac\n"
 				"\t\t%s\n"
 		"	elif test -e \"$path\" -a -r \"$path\"; then\n"
 				"\t\t%s\n"
@@ -1858,6 +1954,7 @@ swc_write_source_copy_script(GB * G,
 		"	;;\n"
 		"	esac\n"
 
+		CSHID
 		"elif test -e \"$sourcepath\" -a -r \"$sourcepath\"; then\n"
 			"\t%s\n"
 			"\t%s\n"
@@ -1866,6 +1963,7 @@ swc_write_source_copy_script(GB * G,
 		"	sw_retval=$?\n"
 			"\t%s\n"
 
+		CSHID
 		"else\n"
 			"\t%s\n"
 			"\t%s\n"
@@ -1878,10 +1976,12 @@ swc_write_source_copy_script(GB * G,
 		"sleep \"$sb__delaytime\"\n"
 		"%s\n"
 		"%s\n"
-			,
+		,
 		SEVENT(2, vlv, SW_SESSION_BEGINS, ""),
 		swicol_subshell_marks(subsh, "source", 'L', nhops, vlv),	
 		strob_str(set_vx),
+		(G->g_do_createM != 0) ? "x" : "",
+		(n_source_specs > 1) ? "x" : "",
 		blocksize,
 		dirname,
 		path,
@@ -1891,6 +1991,8 @@ swc_write_source_copy_script(GB * G,
 		SEVENT(1, -1, SW_SOURCE_ACCESS_BEGINS, ""),
 		SEVENT(2, vlv, SW_SOURCE_ACCESS_BEGINS, ""),
 		pax_write_command,
+		pax_write_command,
+		multiplepathlist ? strob_str(multiplepathlist) : ".",
 		SEVENT(2, vlv, SW_SOURCE_ACCESS_ENDS, "status=0"),
 		strob_str(opt1),
 		SEVENT(1, -1, SW_SOURCE_ACCESS_BEGINS, ""),
@@ -1912,21 +2014,30 @@ swc_write_source_copy_script(GB * G,
 		swicol_subshell_marks(subsh2, "source", 'R', nhops, vlv)
 		);
 
+	xx = strob_str(buffer_new);
+	ret = atomicio((ssize_t (*)(int, void *, size_t))write, ofd, xx, strlen(xx));
+	if (ret != (int)strlen(xx)) {
+		return 1;
+	}
+
+	if (G->g_source_script_name) {
+		swlib_tee_to_file(G->g_source_script_name, -1, xx, -1, 0);
+	}
+
 	free(path);
 	free(dirname);
 	strob_close(tmp);
 	strob_close(tmp1);
 	strob_close(tmp2);
-	if (G->g_source_script_name) {
-		swlib_tee_to_file(G->g_source_script_name, -1, strob_str(buffer), -1, 0);
-	}
 	strob_close(buffer);
+	strob_close(buffer_new);
 	strob_close(opt1);
 	strob_close(opt2);
 	strob_close(set_vx);
 	strob_close(to_devnull);
 	strob_close(subsh);
 	strob_close(subsh2);
+	if (multiplepathlist) strob_close(multiplepathlist);
 	/*
 	 * 0 is OK
 	 * !0 is error
@@ -2473,8 +2584,11 @@ swc_set_shell_dash_s_command(GB * G, char * wopt_shell_command)
 	char * sh_dash_s;
 	
 	tmp = strob_open(32);
-	if (strcasecmp(wopt_shell_command, "POSIX") == 0) {
-		wopt_shell_command = "sh";
+	if (
+		strcasecmp(wopt_shell_command, SWC_SHELL_KEY_POSIX) == 0 ||
+		strcasecmp(wopt_shell_command, SWC_SHELL_KEY_DETECT) == 0
+	) {
+		wopt_shell_command = SWC_SHELL_KEY_SH;
 	}
 	sh_dash_s = swssh_landing_command(wopt_shell_command, 0 /*opt_no_getconf*/);
 	if (G->g_do_cleanshM == 0) {
@@ -2484,23 +2598,6 @@ swc_set_shell_dash_s_command(GB * G, char * wopt_shell_command)
 		   the _swbis args otherwise the it will kill itself */
 		strob_sprintf(tmp, STROB_NO_APPEND, "%s", sh_dash_s);
 	}
-	/*
-	if (strcasecmp(wopt_shell_command, "posix") == 0) {
-		sh_dash_s = "sh -s";
-	} else if (strcasecmp(wopt_shell_command, "bash") == 0) {
-		sh_dash_s = "bash -s";
-	} else if (strcasecmp(wopt_shell_command, "sh") == 0) {
-		sh_dash_s = "sh -s";
-	} else if (strcasecmp(wopt_shell_command, "ash") == 0) {
-		sh_dash_s = "ash -s";
-	} else if (strcasecmp(wopt_shell_command, "ksh") == 0) {
-		sh_dash_s = "ksh -s";
-	} else if (strcasecmp(wopt_shell_command, "dash") == 0) {
-		sh_dash_s = "dash -s";
-	} else {
-		sh_dash_s = "sh -s";
-	}
-	*/
 
 	G->g_sh_dash_s = strdup(strob_str(tmp));
 	strob_close(tmp);
@@ -2657,3 +2754,87 @@ sw_d_msg(GB * G, char * format, ...)
         va_end(ap);
         return ret;
 }
+
+void
+swc_check_shell_command_key(GB * G, char * cmd)
+{
+	char * wopt_shell_command = cmd;
+	if (
+		strcmp(wopt_shell_command, SWC_SHELL_KEY_BASH) &&
+		strcmp(wopt_shell_command, SWC_SHELL_KEY_SH) &&
+		strcmp(wopt_shell_command, SWC_SHELL_KEY_KSH) &&
+		strcmp(wopt_shell_command, SWC_SHELL_KEY_DETECT) &&
+		strcmp(wopt_shell_command, SWC_SHELL_KEY_POSIX)
+	) {
+		sw_e_msg(G, "illegal shell command: %s \n",
+			wopt_shell_command);
+		exit(1);
+	}
+}
+
+void
+swc_check_all_pax_commands(GB * G)
+{
+	char * command;
+	char * xcmd;
+	char * type;
+
+	type = "write";
+	command = get_opta(G->optaM, SW_E_swbis_local_pax_write_command),
+	xcmd = swc_get_pax_write_command(G->g_pax_write_commands, command, 1, (char*)NULL);
+	if (!xcmd) {sw_e_msg(G, "illegal pax %s command: %s \n", type, command); LCEXIT(1); }
+
+	command = get_opta(G->optaM, SW_E_swbis_remote_pax_write_command);
+	xcmd = swc_get_pax_write_command(G->g_pax_write_commands, command, 1, (char*)NULL);
+	if (!xcmd) {sw_e_msg(G, "illegal pax %s command: %s \n", type, command); LCEXIT(1); }
+
+	type = "read";
+	command = get_opta(G->optaM, SW_E_swbis_remote_pax_read_command);
+	xcmd = swc_get_pax_write_command(G->g_pax_write_commands, command, 1, (char*)NULL);
+	if (!xcmd) {sw_e_msg(G, "illegal pax %s command: %s \n", type, command); LCEXIT(1); }
+
+	command = get_opta(G->optaM, SW_E_swbis_local_pax_read_command);
+	xcmd = swc_get_pax_write_command(G->g_pax_write_commands, command, 1, (char*)NULL);
+	if (!xcmd) {sw_e_msg(G, "illegal pax %s command: %s \n", type, command); LCEXIT(1); }
+}
+
+void
+swc_process_pax_command(GB * G, char * command, char * type, int sw_e_num)
+{
+	char * xcmd;
+
+	if (strcmp(type, "write") == 0) {
+		xcmd = swc_get_pax_write_command(
+			G->g_pax_write_commands,
+			command,
+			G->g_verboseG, (char*)NULL);
+	} else if (strcmp(type, "read") == 0) {
+		xcmd = swc_get_pax_read_command(
+			G->g_pax_read_commands,
+			command,
+			0, 0, (char*)NULL);
+	} else if (strcmp(type, "remove") == 0) {
+		xcmd = swc_get_pax_remove_command(
+			G->g_pax_remove_commands,
+			command, 0, (char*)NULL);
+	} else {
+		xcmd = NULL;
+		sw_e_msg(G, "internal error\n");
+		LCEXIT(1);
+	}
+	if (xcmd == NULL) {
+		sw_e_msg(G, "illegal pax %s command: %s \n",
+			type, command);
+			LCEXIT(1);
+	}
+	set_opta(G->optaM, sw_e_num, command);
+}
+
+char *
+swc_get_default_sh_dash_s(GB * G)
+{
+	if (strcmp(get_opta(G->optaM, SW_E_swbis_shell_command), SWC_SHELL_KEY_DETECT) == 0)
+		return SWSSH_DETECT_POISON_DEFAULT;
+	else
+		return G->g_sh_dash_s;	
+}
