@@ -112,6 +112,8 @@
 
    int swc_process_selection_args(VPLOB * , char ** argvector, int uargc, int * poptind);
 
+   int swc_process_selection_files(GB * G, VPLOB * swspec_list);
+
    void swc_flush_logger(GB * G, STROB * slinebuf, STROB * tlinebuf, int efd,
 			int ofd, struct sw_logspec *, int verboselevel);
 
@@ -197,4 +199,9 @@ int sw_d_msg(GB * G, char * format, ...);
 int sw_msg(GB * G, int fd, char * format, ...);
 void swc_stderr_fd2_set(GB * G, int fd);
 void swc_stderr_fd2_restore(GB * G);
+void swc_process_pax_command(GB * G, char * command, char * type, int sw_e_num);
+void swc_check_all_pax_commands(GB * G);
+void swc_check_shell_command_key(GB * G, char * cmd);
+char * swc_get_default_sh_dash_s(GB * G);
+
 #endif
