@@ -71,10 +71,21 @@ abort_script(SWICOL * swicol, GB * G, int fd)
 	swicol_set_master_alarm(swicol);
 	swpl_send_abort(swicol, fd, G->g_swi_event_fd, "");
 }
-		
+
 static
 int
-determine_ls_list_flags(int attr, struct new_cpio_header * file_hdr)
+set_ls_list_flags(void)
+{
+	int ls_verbose;
+	ls_verbose = 0;
+	ls_verbose &= ~(LS_LIST_VERBOSE_WITH_MD5|LS_LIST_VERBOSE_WITH_SHA1|LS_LIST_VERBOSE_WITH_SHA512);
+	ls_verbose &= ~(LS_LIST_VERBOSE_WITH_SIZE|LS_LIST_VERBOSE_WITH_ALL_DATES);
+	return ls_verbose;
+}
+
+static
+int
+determine_ls_list_flags(int attr, struct new_cpio_header * file_hdr, int check_contents)
 {
 
 	FILE_DIGS * digs;
@@ -93,14 +104,24 @@ determine_ls_list_flags(int attr, struct new_cpio_header * file_hdr)
 		ls_verbose |= LS_LIST_VERBOSE_WITH_SYSTEM_NAMES;
 	}
 
+	if (check_contents == 0) {
+		ls_verbose &= ~(LS_LIST_VERBOSE_WITH_MD5|LS_LIST_VERBOSE_WITH_SHA1|LS_LIST_VERBOSE_WITH_SHA512);
+		ls_verbose &= ~(LS_LIST_VERBOSE_WITH_SIZE);
+	}
+
 	digs = file_hdr->digsM;
 	if (digs) {
-		if (digs->do_md5)
+		if (digs->do_md5 == DIGS_ENABLE_ON ) {
 			ls_verbose |= LS_LIST_VERBOSE_WITH_MD5;
-		if (digs->do_sha1)
+		}
+		if (digs->do_sha1 == DIGS_ENABLE_ON ) {
 			ls_verbose |= LS_LIST_VERBOSE_WITH_SHA1;
-		if (digs->do_sha512)
+		}
+		if (digs->do_sha512 == DIGS_ENABLE_ON ) {
 			ls_verbose |= LS_LIST_VERBOSE_WITH_SHA512;
+		}
+	} else {
+		ls_verbose &= ~(LS_LIST_VERBOSE_WITH_MD5|LS_LIST_VERBOSE_WITH_SHA1|LS_LIST_VERBOSE_WITH_SHA512);
 	}
 	return ls_verbose;
 }
@@ -146,87 +167,6 @@ determine_here_document_stop_word(SWI_FILELIST * fl, STROB * buf)
 	return 1; /* error */
 }
 
-int
-swpl_retrieve_files(GB * G, SWI * swi, SWICOL * swicol, SWICAT_E * e, SWI_FILELIST * file_list,
-	int ofd, int ifd, int archive_fd, char * pax_write_command_key, int ls_fd, int ls_verbose, FILE_DIGS * digs)
-{
-	TARU * taru;
-	STROB * scriptbuf;
-	int ret;
-
-	scriptbuf = strob_open(300);
-
-	strob_sprintf(scriptbuf, STROB_DO_APPEND, "rm_retval=1\n");
-	strob_sprintf(scriptbuf, STROB_DO_APPEND, "sw_retval=1\n");
-
-	/* Every task script read stdin even if it doesn't need to */	
-	/* Add the requisite dd bs=512 count=1 of=/dev/null */
-
-	E_DEBUG("");
-	strob_sprintf(scriptbuf, STROB_DO_APPEND,
-		"dd bs=512 count=1 of=/dev/null 2>/dev/null\n"
-		"sw_retval=$?\n"
-		);
-
-	E_DEBUG("");
-
-	swpl_make_verify_command_script_fragment(G, scriptbuf,
-			file_list, pax_write_command_key, ls_fd < 0);
-
-	E_DEBUG("");
-	ret = swicol_rpsh_task_send_script2(
-		swicol,
-		ofd,
-		512,
-		".",
-		strob_str(scriptbuf),
-		SWBIS_TS_retrieve_files_archive);
-
-	if (ret != 0) {
-		/* Set master_alarm ?? */
-		swicol_set_master_alarm(swicol);
-		return -1;
-	}
-
-	/* Send the gratuitous data block because 'count=0' is busted for
-	   some 'dd' programs */
-
-	ret = etar_write_trailer_blocks(NULL, ofd, 1);
-	if (ret <= 0) {
-		return -1;
-	}
-
-	/* Now we have to read the archive */
-
-	E_DEBUG("");
-	taru = taru_create();
-	E_DEBUG("");
-
-	/* here, the archive is read.  This is a tar archive of the
-	   package files as lifted from the file system */
-
-	ret = taru_process_copy_out(taru, ifd, archive_fd,
-			NULL, NULL, arf_ustar, ls_fd, ls_verbose, (intmax_t*)NULL, digs);
-
-	/*
-	ret = swlib_pump_amount(archive_fd, ifd, 4608);
-	fprintf(stderr, "JLJL [%d]\n", ret);
-	*/
-
-	taru_delete(taru);
-	if (ret < 0) {
-		swicol_set_master_alarm(swicol);
-	}
-
-	E_DEBUG("");
-	ret = swicol_rpsh_task_expect(swicol, G->g_swi_event_fd, SWICOL_TL_4);
-
-	E_DEBUG("");
-	strob_close(scriptbuf);
-	E_DEBUG("");
-	return ret;
-}
-
 static
 int
 construct_script_cases(GB * G, STROB * buf, SWI * swi, 
@@ -272,6 +212,7 @@ construct_script_cases(GB * G, STROB * buf, SWI * swi,
 	strob_sprintf(tmp3, 0, "%s: status=$script_retval", script_tag);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
+		CSHID
 		"	# Start of construct_script_cases\n"
 		"	case \"$sw_retval\" in\n"
 		"		" SWBIS_STATUS_COMMAND_NOT_FOUND ")\n"
@@ -281,6 +222,7 @@ construct_script_cases(GB * G, STROB * buf, SWI * swi,
 		"			;;\n"
 					
 		"		%d)\n"
+		"			sw_retval=0\n"
 		"			;;\n"
 					/* Do nothing */
 		"		%d)\n"
@@ -311,6 +253,162 @@ construct_script_cases(GB * G, STROB * buf, SWI * swi,
 	return 0;
 }
 
+static
+int
+form_entry_names(GB * G, SWICAT_E * e, char * iscpath, STROB * tmp_epath, STROB * tmp_depath)
+{
+	char * epath; 
+	char * depath;
+	char * s_ret1; 
+	char * s_ret2; 
+	int retval;
+	
+	retval = 0;
+
+	E_DEBUG("Entering");
+	s_ret1 = swicat_e_form_catalog_path(e, tmp_epath,
+			iscpath /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
+			SWICAT_ACTIVE_ENTRY);
+	epath = strob_str(tmp_epath);
+
+	s_ret2 = swicat_e_form_catalog_path(e, tmp_depath,
+			iscpath /*get_opta(G->optaM, SW_E_installed_software_catalog) */,
+			SWICAT_DEACTIVE_ENTRY);
+	depath = strob_str(tmp_depath);
+
+	if (swlib_check_clean_relative_path(epath) != 0) {
+		sw_e_msg(G, "tainted path name: %s\n", epath);
+		strob_strcpy(tmp_epath, "");
+		s_ret1 = NULL;
+	}
+	
+	if (swlib_check_clean_relative_path(depath) != 0) {
+		sw_e_msg(G, "tainted path name: %s\n", depath);
+		strob_strcpy(tmp_depath, "");
+		s_ret2 = NULL;
+	}
+
+	if (s_ret1 == NULL || s_ret2 == NULL) {
+		return -1;
+	}
+
+	return 0;
+}
+
+static
+int
+common_remove_catalog_entry(GB * G, SWICOL * swicol, int ofd, char * epath, char * depath, char * task_id)
+{
+	STROB * btmp;
+	STROB * dir2;
+	int retval;
+	int ret;
+	
+	retval = 0;
+	btmp = strob_open(32);
+	dir2 = strob_open(32);
+
+	strob_strcpy(dir2, epath);
+	swlib_unix_dirtrunc_n(dir2, 2);
+	
+	if (strcmp(task_id, SWBIS_TS_remove_catalog_entry) == 0) {
+		strob_sprintf(btmp, 0,
+			CSHID
+			"	# Remove all old catalog entries\n"
+			" 	(cd \"%s\" && find . -type d -name '_[0-9]*' -exec rm -fr {} \\; 2>/dev/null ) </dev/null \n",
+			strob_str(dir2)
+		);
+	}
+
+	strob_sprintf(btmp, 1,
+		CSHID
+		"	rm -fr \"%s\" 1>&2\n"
+		"	mv -f \"%s\" \"%s\" 1>&2\n"
+		"	sw_retval=$?\n"
+		" 	(cd \"%s\" && rmdir * 2>/dev/null ) </dev/null \n"
+		"	case $sw_retval in 0) ;; *) sw_retval=2 ;; esac\n"
+		"	dd count=1 bs=512 of=/dev/null 2>/dev/null\n",
+		depath,
+		epath,
+		depath,
+		strob_str(dir2)
+		);
+
+	ret = swicol_rpsh_task_send_script2(
+		swicol,
+		ofd, 		/* file descriptor */
+		512,		/* null dummy block */
+		".",   		/* i.e. stay in the target path */
+		strob_str(btmp),/* The actual task script */
+		task_id
+		);
+	if (ret < 0) {
+		retval++;
+		goto error_out;
+	}
+
+	ret = etar_write_trailer_blocks(NULL, ofd, 1);
+	if (ret < 0) {
+		retval++;
+		goto error_out;
+	}
+
+	ret = swicol_rpsh_task_expect(swicol, G->g_swi_event_fd,
+			SWICOL_TL_9 /*time limit*/);
+
+	if (ret < 0) {
+		retval = ret;
+		SWLIB_INTERNAL("");
+		goto error_out;
+	}
+
+	if (ret > 0) {
+		retval = ret;
+	}
+
+	error_out:
+
+	strob_close(btmp);
+	strob_close(dir2);
+	return retval;
+}
+
+
+static int
+alter_catalog_entry(int remove_restore, GB * G, SWICOL * swicol, SWICAT_E * e, int ofd, char * iscpath)
+{
+	STROB * btmp;
+	STROB * tmp;
+	STROB * tmp2;
+	STROB * dir2;
+	char * epath; 
+	char * depath;
+	int retval;
+	int ret;
+	
+	retval = 0;
+	tmp = strob_open(32);
+	tmp2 = strob_open(32);
+
+	ret = form_entry_names(G, e, iscpath, tmp, tmp2);
+	if (ret)
+		return 1;
+
+	epath = strob_str(tmp);
+	depath = strob_str(tmp2);
+	if (remove_restore == 0)
+		ret = common_remove_catalog_entry(G, swicol, ofd, epath, depath, SWBIS_TS_remove_catalog_entry);
+	else
+		ret = common_remove_catalog_entry(G, swicol, ofd, depath, epath, SWBIS_TS_restore_catalog_entry);
+
+	if (ret)
+		return 2;
+
+	strob_close(tmp);
+	strob_close(tmp2);
+	return retval;
+}
+
 int
 swpl_send_null_task(SWICOL * swicol, int ofd, int event_fd, char * msgtag, int retcode)
 {
@@ -344,6 +442,7 @@ swpl_send_null_task2(SWICOL * swicol, int ofd, int event_fd, char * msgtag, char
 	 * Here is the minimal task scriptlet that throws an error
 	 */
 	strob_sprintf(tmp, 0,
+		CSHID
 		"# sw_retval=%%d\n"
 		"%s\n"
 		"sleep 0\n"
@@ -504,6 +603,86 @@ swpl_scary_find_script(SCAR ** array, char * tag, char * tagspec)
 	return retval;
 }
 
+int
+swpl_retrieve_files(GB * G, SWI * swi, SWICOL * swicol, SWICAT_E * e, SWI_FILELIST * file_list,
+	int ofd, int ifd, int archive_fd, char * pax_write_command_key, int ls_fd, int ls_verbose, FILE_DIGS * digs)
+{
+	TARU * taru;
+	STROB * scriptbuf;
+	STROB * shell_lib_buf;
+	int ret;
+
+	scriptbuf = strob_open(300);
+	shell_lib_buf = strob_open(300);
+
+	strob_sprintf(scriptbuf, STROB_DO_APPEND, "rm_retval=1\n");
+	strob_sprintf(scriptbuf, STROB_DO_APPEND, "sw_retval=1\n");
+			
+	/* Every task script read stdin a block from even if it doesn't
+           need data */	
+
+	/* Add the requisite dd bs=512 count=1 of=/dev/null */
+	E_DEBUG("");
+	strob_sprintf(scriptbuf, STROB_DO_APPEND,
+		"dd bs=512 count=1 of=/dev/null 2>/dev/null\n"
+		"sw_retval=$?\n"
+		);
+
+	E_DEBUG("");
+
+	swpl_make_verify_command_script_fragment(G, scriptbuf,
+			file_list, pax_write_command_key, ls_fd < 0);
+
+	E_DEBUG("");
+	ret = swicol_rpsh_task_send_script2(
+		swicol,
+		ofd,
+		512,
+		".",
+		strob_str(scriptbuf),
+		SWBIS_TS_retrieve_files_archive);
+
+	if (ret != 0) {
+		/* Set master_alarm ?? */
+		swicol_set_master_alarm(swicol);
+		return -1;
+	}
+
+	/* Send the gratuitous data block because 'count=0' is busted for
+	   some 'dd' programs */
+
+	ret = etar_write_trailer_blocks(NULL, ofd, 1);
+	if (ret <= 0) {
+		return -1;
+	}
+
+	/* Now we have to read the archive */
+
+	E_DEBUG("");
+	taru = taru_create();
+	E_DEBUG("");
+
+	/* here, the archive is read.  This is a tar archive of the
+	   package files as lifted from the file system */
+
+	ret = taru_process_copy_out(taru, ifd, archive_fd,
+			NULL, NULL, arf_ustar, ls_fd, ls_verbose, (intmax_t*)NULL, digs);
+
+	taru_delete(taru);
+	if (ret < 0) {
+		swicol_set_master_alarm(swicol);
+	}
+
+	E_DEBUG("");
+	ret = swicol_rpsh_task_expect(swicol, G->g_swi_event_fd, SWICOL_TL_4);
+
+	E_DEBUG("");
+	strob_close(scriptbuf);
+	strob_close(shell_lib_buf);
+	E_DEBUG("");
+	return ret;
+}
+
 void
 swpl_audit_execution_scripts(GB * G, SWI * swi, SCAR ** scary_scripts)
 {
@@ -582,6 +761,7 @@ swpl_write_control_script_code_fragment(GB * G, SWI * swi, SWI_CONTROL_SCRIPT *
 		script->baseM.tagM);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
+		CSHID
 		"	# Start of swpl_write_control_script_code_fragment\n"
 		"	cd \"%s\" || exit 44\n"
 		"	cd \"%s\" || exit 45\n"
@@ -623,6 +803,7 @@ swpl_write_case_block(SWI * swi, STROB * buf, char * tag)
 	swicat_write_script_cases(swi, tmp, tag);
 
 	strob_sprintf(buf, STROB_DO_APPEND,
+		CSHID
 		"# generated by swpl_write_case_block\n"
 		"	%s)\n"
 		"%s"
@@ -862,6 +1043,7 @@ swpl_construct_configure_script(GB * G, STROB * buf, SWI * swi, int do_configure
 {
 	int ret;
 	strob_sprintf(buf, STROB_DO_APPEND,
+		CSHID
 		"	# Start of code generated by swpl_construct_configure_script \n"
 		"		ssv_do_configure=%d\n"
 		"		case \"$ssv_do_configure\" in\n"
@@ -964,6 +1146,7 @@ swpl_construct_script(GB * G, STROB * buf, SWI * swi, char * script_tag)
 		 * construct the product script's execution
 		 */
 		strob_sprintf(toap, STROB_DO_APPEND, 
+			CSHID
 			"# Start of code generated by swproglib.c:swpl_construct_script\n",
 			"cd \"%s\"\n",
 			target_path);
@@ -984,6 +1167,7 @@ swpl_construct_script(GB * G, STROB * buf, SWI * swi, char * script_tag)
 
 	if (fileset_xxinstall_script) {
 		strob_sprintf(toap, STROB_DO_APPEND,
+			CSHID
 		"	case \"$sw_retval\" in  # Case_construct_script_003\n"
 		"	0)\n"
 		);
@@ -1020,10 +1204,11 @@ swpl_construct_analysis_script(GB * G, STROB * buf, SWI * swi, SWI_CONTROL_SCRIP
 	struct extendedOptions * opta;
 	int checkscript_retcode;
 	int swi_retcode;
+	int check_def_retcode;
 
 	opta = swi->optaM;
 
-	prod = swi_package_get_product(swi->swi_pkgM, 0 /* The first product */);
+	prod = swi_package_get_product(swi->swi_pkgM, 0 /* FIXME The first product */);
 	if (swi_product_has_control_file(prod, SW_A_checkinstall)) {
 	
 		strob_sprintf(buf, STROB_DO_APPEND,
@@ -1052,32 +1237,54 @@ swpl_construct_analysis_script(GB * G, STROB * buf, SWI * swi, SWI_CONTROL_SCRIP
 			swi_retcode = 0;
 		}
 
+		if (G->g_force) {
+			check_def_retcode = 0;
+		} else {
+			check_def_retcode = SW_DESELECT;
+		}
+
 		*p_script = script;
 		strob_sprintf(buf, STROB_DO_APPEND,
-		"# Generated by swproglib.c:construct_analysis script\n"
+		CSHID
 		"case \"$sw_retval\" in\n"
-		"	%d)\n"
+		"	%d)\n"   /* SW_SUCCESS */
+		"		sw_retval=0\n"
 		"		;;\n"
 				/* Do nothing */
-		"	%d)\n"
+		"	%d)\n"  /* SW_WARNING */
+		"		sw_retval=0\n"
+		"		%s\n"
+		"		;;\n"
+
+		"	%d)\n"  /* SW_ERROR */
+		"		script_retval=%d\n"
+		"		sw_retval=%d\n"
 		"		%s\n"
 		"		sw_retval=0\n"
 		"		;;\n"
-
-		"	%d)\n"
+		"	%d)\n"  /* SW_DESELECT */
 		"		script_retval=%d\n"
-		"		%s\n"
 		"		sw_retval=%d\n"
+		"		%s\n"
+		"		sw_retval=0\n"
+		"		;;\n"
+		"	*)\n"
+		"		sw_retval=4\n" /* Big error */
 		"		;;\n"
 		"esac\n"
 		"dd of=/dev/null 2>/dev/null\n",
+
 		SW_SUCCESS,
 		SW_WARNING,
 		TEVENT(2, (swi->verboseM), SW_CHECK_SCRIPT_WARNING, "status=$sw_retval"),
 		SW_ERROR,
 		checkscript_retcode,
+		swi_retcode,
 		TEVENT(2, (swi->verboseM), SW_CHECK_SCRIPT_ERROR, "status=$script_retval"),
-		swi_retcode
+		SW_DESELECT,
+		SW_DESELECT,
+		check_def_retcode,
+		TEVENT(2, (swi->verboseM), SW_CHECK_SCRIPT_EXCLUDE, "status=$sw_retval")
 		);	
 	} else {
 		*p_script = NULL;
@@ -1116,7 +1323,6 @@ void
 swpl_sanitize_pathname(char * s)
 {
 	swlib_squash_all_leading_slash(s);
-	/* swpl_safe_check_pathname(s); */
 }
 
 char * 
@@ -1606,6 +1812,7 @@ swpl_send_signature_files(SWI * swi, int ofd,
 	 * Here is the task scriptlet.
 	 */
 	strob_sprintf(tmp, 0,
+		CSHID
 		"dd 2>/dev/null | %s\n"
 		"sw_retval=$?\n"
 		"dd of=/dev/null 2>/dev/null\n"
@@ -1777,6 +1984,7 @@ swpl_update_execution_script_results(SWI * swi, SWICOL * swicol, SCAR ** array)
 	int event_index = -1;
 	int event_start_index;
 	int status;
+	char * savechp;
 	char * tag;
 	char * tagspec;
 	char * message;
@@ -1784,8 +1992,11 @@ swpl_update_execution_script_results(SWI * swi, SWICOL * swicol, SCAR ** array)
 	buf = strob_open(100);
 	tmp = strob_open(100);
 
-	event_start_index = swicol->event_indexM;
+	E_DEBUG("");
+	/* event_start_index = swicol->event_indexM; */
+	event_start_index = 0;
 
+	E_DEBUG2("event_start_index=%d", event_start_index);
 	/*
 	swicol_print_events(swicol, buf, -1);	
 	fprintf(stderr, "%s", strob_str(buf));
@@ -1801,10 +2012,12 @@ swpl_update_execution_script_results(SWI * swi, SWICOL * swicol, SCAR ** array)
 	while (message) {
 		/* fprintf(stderr, "message=[%s]\n", message);
 			*/
+		E_DEBUG2("message=%s", message);
 		tagspec = message;
 		tag = strchr(message, (int)(' '));
 		SWLIB_ASSERT(tag != NULL);
-		*tag='\0';
+		savechp = tag;
+		*tag = '\0';
 		tag++;
 		/* fprintf(stderr, "tag=[%s] tagspec=[%s]\n", tag, tagspec);
 			*/
@@ -1812,15 +2025,19 @@ swpl_update_execution_script_results(SWI * swi, SWICOL * swicol, SCAR ** array)
 		 * Now find the script that belongs to this tag and tagspec
 		 */
 		script = swpl_scary_find_script(array, tag, tagspec);
+		E_DEBUG2("script tag=%s", tag);
+		E_DEBUG2("script tagspec=%s", tagspec);
 		SWLIB_ASSERT(script != NULL);
 		status = swicol_rpsh_get_event_status(swicol, (char*)(NULL),
                 		SW_CONTROL_SCRIPT_ENDS, event_index, &event_index);
 		SWLIB_ASSERT(status >= 0);
 		script->resultM = status;
+		E_DEBUG2("script result=%d", status);
 		message = swicol_rpsh_get_event_message(swicol,
 				SW_CONTROL_SCRIPT_BEGINS,
 				event_index+1,
 				&event_index);
+		*savechp = (int)(' ');		
 	}
 
 
@@ -1841,6 +2058,7 @@ swpl_unpack_catalog_tarfile(SWI * swi, int ofd,
 	E_DEBUG("");
 
 	strob_sprintf(tmp, 0,
+		CSHID
 		"#set -vx\n"
 		"# echo unpacking catalog %s 1>&2\n"
 		"dd of=/dev/null 2>/dev/null\n"
@@ -1927,6 +2145,7 @@ swpl_remove_catalog_directory(SWI * swi, int ofd,
 	}
 
 	strob_sprintf(tmp, 0,
+		CSHID
 		"cd " SWINSTALL_INCAT_NAME "\n"
 		"/bin/rm -fr \"%s\" %s\n"
 		"sw_retval=$?\n"
@@ -1960,6 +2179,7 @@ swpl_get_utsname_attributes(GB * G, SWICOL * swicol, SWUTS * uts, int ofd, int e
 	STROB * tmp1 = strob_open(10);
 
 	strob_sprintf(tmp, 0,
+			CSHID
 			"sw_retval=0\n"
 			"%s\n" /* shls_config_guess() */
 			"%s\n" /* shls_run_config_guess() */
@@ -2169,9 +2389,6 @@ swpl_get_dependency_specs(GB * G, SWI * swi, char * requisite_keyword, int produ
 	VPLOB * swspecs;
 	SWI_PRODUCT * product;
 	SWI_XFILE * fileset;
-	char * next_line;
-	char * keyword;
-	char * missing_name;
 	SWHEADER * global_index;
 	SWVERID * swverid;
 	STROB * tmp;
@@ -2233,7 +2450,6 @@ swpl_get_dependency_specs(GB * G, SWI * swi, char * requisite_keyword, int produ
 				fprintf(stderr, "%s\n", strob_str(tmp2));
 				*/
 			}
-			/* fprintf(stderr, "JL [%s]\n", depspec); */
 			depspec = strob_strtok(tmp, NULL, " \n\r\t");
 		}
 		list++;
@@ -2262,9 +2478,6 @@ swpl_get_catalog_entries(GB * G,
 	STROB * target_catalog_dir;
 	STROB * isc_script_buf;
 	int aa_fd;
-	int aa_len;
-	int aa_len2;
-	int aa_i;
 	int ret;
 	int retval;
 	
@@ -2550,7 +2763,7 @@ swpl_get_catalog_tar(GB * G,
 	while ((path=swvarfs_get_next_dirent(swvarfs, &st)) != NULL && strlen(path)) {
 		E_DEBUG("");
 		if (G->devel_verboseM)
-			fprintf(stderr, "JL path=[%s]\n", path);
+			fprintf(stderr, "<debug>: path=[%s]\n", path);
 	}
 
 	E_DEBUG("");
@@ -2560,7 +2773,7 @@ swpl_get_catalog_tar(GB * G,
 	while ((path=swvarfs_get_next_dirent(swvarfs, &st)) != NULL && strlen(path)) {
 		E_DEBUG3("catalog_tar: %s: %s", strob_str(target_catalog_dir), path);
 		if (G->devel_verboseM)
-			fprintf(stderr, "JL path=[%s]\n", path);
+			fprintf(stderr, "<debug> path=[%s]\n", path);
 	}
 	E_DEBUG("");
 	swvarfs_close(swvarfs);
@@ -2598,7 +2811,6 @@ swpl_get_samepackage_query_response(GB * G, SWICOL * swicol,
 			int * p_retval,
 			int make_dummy_response)
 {
-	int ret;
 	int uxfio_fd;
 	char * mem;
 
@@ -2648,12 +2860,8 @@ SWICAT_REQ *
 swpl_analyze_samepackage_query_response(GB * G, char * response_image, SWICAT_SL ** p_sl)
 {
 	int ret;
-	int i;
-	int j;
 	SWICAT_REQ  * req;
 	SWICAT_SL * sl;
-	SWICAT_SC * sc;
-	SWICAT_SR * sr;
 
 	if (response_image == NULL) return NULL;
 	req = swicat_req_create();
@@ -2692,6 +2900,7 @@ swpl_shellfrag_session_lock(STROB * buf, int vlv)
 	 */
 
 	strob_sprintf(buf, 0,
+	CSHID
 	"#\n"
 	"#  <<<<----- here's the code to lock a session \n"
 	"#\n"
@@ -2787,6 +2996,7 @@ char *
 swpl_shellfrag_session_unlock(STROB * buf, int vlv)
 {
 	strob_sprintf(buf, 0,
+	CSHID
 	"case \"$lock_did_lock\" in\n"
 	"       \"\") ;;\n"
 	"       *)\n" 
@@ -2812,6 +3022,7 @@ swpl_session_lock(GB * G, SWICOL * swicol, char * target_path, int ofd, int even
 
 	/* swicol_set_task_idstring(swicol, SWBIS_TS_make_locked_session); */
 	strob_sprintf(tmp, 0,
+		CSHID
 		"%s\n"
 		"%s\n"
 		"%s\n"
@@ -2874,7 +3085,6 @@ swpl_session_lock(GB * G, SWICOL * swicol, char * target_path, int ofd, int even
 char *
 swpl_looper_payload_routine(STROB * buf, int vlv, char * locked_region)
 {
-	char * ret;
 	STROB * lockfrag_buffer;
 	STROB * unlockfrag_buffer;
 
@@ -2882,6 +3092,7 @@ swpl_looper_payload_routine(STROB * buf, int vlv, char * locked_region)
 	unlockfrag_buffer = strob_open(164);
 
 	strob_sprintf(buf, 0,
+		CSHID
 		"shls_looper_payload()\n"
 		"{\n"
 		"	catalog_entry_dir=\"$1\"\n"
@@ -2948,8 +3159,6 @@ swpl_signature_policy_accept(GB * G, SWGPG_VALIDATE * w, int verbose_level, char
 	int gotta_have_all;
 	struct extendedOptions * opta;
 	char * blob;
-	char * name;
-	char * sig;
 	int result;
 	int sig_level;
 	int sig_fd;
@@ -3060,124 +3269,16 @@ swpl_signature_policy_accept(GB * G, SWGPG_VALIDATE * w, int verbose_level, char
 int
 swpl_remove_catalog_entry(GB * G, SWICOL * swicol, SWICAT_E * e, int ofd, char * iscpath)
 {
-	STROB * btmp;
-	STROB * tmp;
-	STROB * tmp2;
-	STROB * dir2;
-	char * epath; 
-	char * depath;
-	char * s_ret1; 
-	char * s_ret2; 
 	int retval;
-	int ret;
-	
-	retval = 0;
-	tmp = strob_open(32);
-	tmp2 = strob_open(32);
-	btmp = strob_open(32);
-	dir2 = strob_open(32);
+	retval = alter_catalog_entry(0 /*remove_restore*/, G, swicol, e, ofd, iscpath);
+	return retval;
+}
 
-	E_DEBUG("Entering");
-	s_ret1 = swicat_e_form_catalog_path(e, tmp,
-			iscpath /* get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_ACTIVE_ENTRY);
-	epath = strob_str(tmp);
-
-	/* SWLIB_INFO2("catalog entry (active) path = %s", epath); */
-	s_ret2 = swicat_e_form_catalog_path(e, tmp2,
-			iscpath /*get_opta(G->optaM, SW_E_installed_software_catalog) */,
-			SWICAT_DEACTIVE_ENTRY);
-	depath = strob_str(tmp2);
-
-	/* SWLIB_INFO2("catalog entry (active) path = %s", depath); */
-
-	if (swlib_check_clean_relative_path(epath) != 0) {
-		sw_e_msg(G, "tainted path name: %s\n", epath);
-		epath = NULL;
-	}
-	
-	if (swlib_check_clean_relative_path(depath) != 0) {
-		sw_e_msg(G, "tainted path name: %s\n", depath);
-		depath = NULL;
-	}
-
-	if (s_ret1 == NULL || s_ret2 == NULL) {
-		SWLIB_INFO("error: entry not removed: test, test");
-		ret = swpl_send_nothing_and_wait(swicol, ofd, G->g_swi_event_fd,
-			SWBIS_TS_remove_catalog_entry,
-			/* wait this long */ SWICOL_TL_8,
-			/* exit with this return status */ SW_SUCCESS);
-		if (ret < 0) retval++;
-		goto error_out;
-	}
-	
-	E_DEBUG2("epath=[%s]", epath);
-	E_DEBUG2("depath=[%s]", depath);
-	E_DEBUG2("e->entry_prefix=[%s]", e->entry_prefixM);
-
-	/* For example
-	DEBUG:         swproglib.c:2907  , swpl_remove_catalog_entry : epath=[var/lib/swbis/catalog/swbis/swbis/0.730/0]
-	DEBUG:         swproglib.c:2908  , swpl_remove_catalog_entry : depath=[var/lib/swbis/catalog/swbis/swbis/0.730/_0]
-	*/
-
-	strob_strcpy(dir2, epath);
-	swlib_unix_dirtrunc_n(dir2, 2);
-
-	strob_sprintf(btmp, 0,
-		"	# Remove all old catalog entries\n"
-		" 	(cd \"%s\" && find . -type d -name '_[0-9]*' -exec rm -fr {} \\; 2>/dev/null ) </dev/null \n"
-		"	rm -fr \"%s\" 1>&2\n"
-		"	mv -f \"%s\" \"%s\" 1>&2\n"
-		"	sw_retval=$?\n"
-		" 	(cd \"%s\" && rmdir * 2>/dev/null ) </dev/null \n"
-		"	case $sw_retval in 0) ;; *) sw_retval=2 ;; esac\n"
-		"	dd count=1 bs=512 of=/dev/null 2>/dev/null\n",
-		strob_str(dir2),
-		depath,
-		epath,
-		depath,
-		strob_str(dir2)
-		);
-
-	/* swicol_set_task_idstring(swicol, SWBIS_TS_remove_catalog_entry); */
-	ret = swicol_rpsh_task_send_script2(
-		swicol,
-		ofd, 		/* file descriptor */
-		512,		/* null dummy block */
-		".",   		/* i.e. stay in the target path */
-		strob_str(btmp),/* The actual task script */
-		SWBIS_TS_remove_catalog_entry
-		);
-	if (ret < 0) {
-		retval++;
-		goto error_out;
-	}
-
-	ret = etar_write_trailer_blocks(NULL, ofd, 1);
-	if (ret < 0) {
-		retval++;
-		goto error_out;
-	}
-
-	ret = swicol_rpsh_task_expect(swicol, G->g_swi_event_fd,
-			SWICOL_TL_9 /*time limit*/);
-
-	if (ret < 0) {
-		retval = ret;
-		SWLIB_INTERNAL("");
-		goto error_out;
-	}
-
-	if (ret > 0) {
-		retval = ret;
-	}
-
-	error_out:
-
-	strob_close(tmp);
-	strob_close(tmp2);
-	strob_close(btmp);
-	strob_close(dir2);
+int
+swpl_restore_catalog_entry(GB * G, SWICOL * swicol, SWICAT_E * e, int ofd, char * iscpath)
+{
+	int retval;
+	retval = alter_catalog_entry(1 /*remove_restore*/, G, swicol, e, ofd, iscpath);
 	return retval;
 }
 
@@ -3259,9 +3360,12 @@ swpl_make_verify_command_script_fragment(GB * G,
 {
 	int ret;
 	STROB * stop_word;
+	STROB * shell_lib_buf;
 	char * name;
 	int ix;
 	
+	shell_lib_buf = strob_open(24);
+
 	/* Tag every file in a volatile directory as being volatile itself
 	   FIXME ??? is this the correct thing to do */
 
@@ -3294,6 +3398,56 @@ swpl_make_verify_command_script_fragment(GB * G,
 		) | tar  cf - --no-recursion --files-from=-
 	*/
 
+	strob_sprintf(scriptbuf, STROB_DO_APPEND,
+		"%s\n",
+		shlib_get_function_text_by_name("shls_check_for_gnu_tar", shell_lib_buf, NULL)
+		);
+
+	strob_sprintf(scriptbuf, STROB_DO_APPEND,
+		"%s\n"
+		"%s\n",
+		shlib_get_function_text_by_name("shls_missing_which", shell_lib_buf, NULL),
+		shlib_get_function_text_by_name("shls_write_files_ar", shell_lib_buf, NULL)
+		);
+
+	if (strcmp(pax_write_command_key, "gtar") == 0 ||
+	    strcmp(pax_write_command_key, "pax") == 0) {
+		strob_sprintf(scriptbuf, STROB_DO_APPEND,
+			CSHID
+			"has_gnu_tar=0\n");
+	} else if (strcmp(pax_write_command_key, "tar") == 0) {
+		/* Check to see if tar is GNU tar */
+		
+		strob_sprintf(scriptbuf, STROB_DO_APPEND,
+			CSHID
+			"shls_check_for_gnu_tar\n"
+			"has_gnu_tar=$?\n");
+	} else if (strcmp(pax_write_command_key, "detect") == 0) {
+		/* Use GNU tar if tar is GNU tar, or use pax
+		   this is handled below */
+		strob_sprintf(scriptbuf, STROB_DO_APPEND,
+			CSHID
+			"has_gnu_tar=0\n");
+	} else {
+		strob_sprintf(scriptbuf, STROB_DO_APPEND,
+			CSHID
+			"has_gnu_tar=0\n");
+	}
+
+	strob_sprintf(scriptbuf, STROB_DO_APPEND,
+		"\n"
+		"case $has_gnu_tar in\n"
+		"	0)\n"
+		"	;;\n"
+		"	*)\n"
+		"	# simulate the response of GNU tar to an empty list\n"
+		"	dd count=2 bs=512 if=/dev/zero 2>/dev/null \n"
+		"	exit " xSTR(SWP_RP_STATUS_NO_GNU_TAR) "\n"
+		"	;;\n"
+		"esac\n"
+		);
+
+
 	/* Start of hear document */
 	strob_sprintf(scriptbuf, STROB_DO_APPEND,
 		"(\n"
@@ -3319,36 +3473,60 @@ swpl_make_verify_command_script_fragment(GB * G,
 		strob_str(stop_word));
 
 
-	/* Sanity check on pax_write_command_key */
-
-	if (strcmp(pax_write_command_key, "tar") == 0) {
-		;
-	} else if (strcmp(pax_write_command_key, "gtar") == 0) {
-		;
-	} else {
-		/* This is an error, should write a warning  */
-		pax_write_command_key = "tar";
-	}
-	
+	E_DEBUG("");
 	/* write the tar writing command */
+	if (
+		strcmp(pax_write_command_key, "tar") == 0 ||
+		strcmp(pax_write_command_key, "gtar") == 0 ||
+		0
+	) {
+		do_tar_key: /* goto label */
 
+		strob_sprintf(scriptbuf, STROB_DO_APPEND,
+			" %s %scf - -b 1 --no-recursion --files-from=- %s\n",
+			pax_write_command_key,
+			(char*)(G->g_verboseG > SWC_VERBOSE_2 ? "v" : ""),
+			(be_silent ? "2>/dev/null" : ""));
+	} else if (
+		strcmp(pax_write_command_key, "pax") == 0 ||
+		0
+	) {
+		strob_sprintf(scriptbuf, STROB_DO_APPEND, " pax -d -w -b 512\n");
+	} else if (
+		strcmp(pax_write_command_key, "detect") == 0 ||
+		0
+	) {
+		/* write code to use either pax or tar, making sure it is GNU tar */
+
+		strob_sprintf(scriptbuf, STROB_DO_APPEND,
+				" shls_write_files_ar\n"
+				CSHID
+				"# Support controlled bailout in the event \n"
+				"# neither GNU tar or pax is present \n"
+				"case $? in 126) exit " xSTR(SWP_RP_STATUS_NO_GNU_TAR) ";; esac\n"
+				);
+	} else {
+		goto do_tar_key;
+	}
+
+	/* FIXME, sw_retval is set above only, and does not report
+	   the exit status of the primary reading command */
 
 	E_DEBUG("");
 	strob_sprintf(scriptbuf, STROB_DO_APPEND,
-				" %s %scf - -b 1 --no-recursion --files-from=- %s\n",
-				pax_write_command_key,
-				(char*)(G->g_verboseG > SWC_VERBOSE_2 ? "v" : ""),
-				(be_silent ? "2>/dev/null" : "")
-				);
-	strob_sprintf(scriptbuf, STROB_DO_APPEND, "retrieve_retval=$?\n");
+			"# FIXME, sw_retval should be set here\n"
+			"### Not used   retrieve_retval=$?\n"
+			CSHID
+			);
 
 	E_DEBUG("");
+	strob_close(shell_lib_buf);
 	strob_close(stop_word);
 	return 0;
 }
 
 int
-swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_attributes, int skip_volatile)
+swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_attributes, int skip_volatile, int check_contents)
 {
 	int retval;
 	SWHEADER_STATE state1;
@@ -3361,6 +3539,9 @@ swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_a
 	char * next_attr;
 	char * keyword;
 	char * value;
+	int has_md5;
+	int has_sha1;
+	int has_sha512;
 	AHS * info_ahs2;
 	int ret;
 	STROB * ls_line;
@@ -3368,11 +3549,20 @@ swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_a
 	FILE_DIGS * digs;
 
 	E_DEBUG("");
+	has_md5 = 0;
+	has_sha1 = 0;
+	has_sha512 = 0;
+
 	file_attribute_usage = 0;
 	retval = 0;
 	info_ahs2 = ahs_open();
 	ls_line = strob_open(60);
 
+	if (check_contents == 0) {		
+		ls_verbose &= ~(LS_LIST_VERBOSE_WITH_MD5|LS_LIST_VERBOSE_WITH_SHA1|LS_LIST_VERBOSE_WITH_SHA512);
+		ls_verbose &= ~(LS_LIST_VERBOSE_WITH_SIZE);
+	}
+
 	/* FIXME: Eventually support multiple products and filesets  */
 
 	swpl_enforce_one_prod_one_fileset(swi);
@@ -3394,7 +3584,8 @@ swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_a
 	file_hdr = ahs_vfile_hdr(info_ahs2);
 	taru_init_header_digs(file_hdr);
 	digs = file_hdr->digsM;
-	taru_digs_init(digs, 1 /*enable*/);
+
+	taru_digs_init(digs, check_contents ? 1 : 0, DIGS_DO_POISON);
 
         while ((next_line=swheader_get_next_object(infoheader, (int)UCHAR_MAX, (int)UCHAR_MAX))) {
 
@@ -3415,7 +3606,7 @@ swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_a
 
 		taru_init_header(file_hdr);
 		file_hdr->digsM = digs;
-		taru_digs_init(digs, 1 /*enable*/);
+		taru_digs_init(digs, check_contents ? 1 : 0, DIGS_DO_POISON);
 		
 		swheader_store_state(infoheader, &state1);
 
@@ -3432,11 +3623,25 @@ swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_a
 			file_attribute_usage &= file_hdr->usage_maskM;
 		}
 
-		if (available_attributes) {
-			/* file_attribute_usage are the TARU_UM_* flags and it returns
-			   the policy in terms of the LS_LIST_VERBOSE flags  */
-			*available_attributes = determine_ls_list_flags(file_attribute_usage, file_hdr);
-			/* fprintf(stderr, "=========== %d\n", *available_attributes); */
+		/* file_attribute_usage are the TARU_UM_* flags and it returns
+		   the policy in terms of the LS_LIST_VERBOSE flags  */
+		*available_attributes = determine_ls_list_flags(file_attribute_usage, file_hdr, check_contents);
+		/* fprintf(stderr, "=========== %d\n", *available_attributes); */
+
+		/* The contents of the INFO object are first detected and set in the
+		   DIGS object in the file_hdr object, then they are transferred to
+		   the *available_attributes flags by determine_ls_list_flags() */
+		
+		if (check_contents) {
+			if (*available_attributes & LS_LIST_VERBOSE_WITH_MD5) { has_md5 = 1; }
+			if (*available_attributes & LS_LIST_VERBOSE_WITH_SHA1) { has_sha1 = 1; }
+			if (*available_attributes & LS_LIST_VERBOSE_WITH_SHA512) { has_sha512 = 1; }
+			
+			/* If any files have a digest then assume all of them do */
+
+			if (has_md5) ls_verbose |= LS_LIST_VERBOSE_WITH_MD5;
+			if (has_sha1) ls_verbose |= LS_LIST_VERBOSE_WITH_SHA1;
+			if (has_sha512) ls_verbose |= LS_LIST_VERBOSE_WITH_SHA512;
 		}
 
 		swheader_restore_state(infoheader, &state1);
@@ -3479,3 +3684,58 @@ swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_a
 	strob_close(ls_line);
 	return retval;
 }
+
+void
+swpl_bashin_detect(STROB * buf)
+{
+	int i;
+	int j;
+	char * prog;
+	char * swbis;
+	
+	/* Here is the code to detect a shell that complies with
+	the POSIX prescribed handing of STDIN which is not to
+	read ahead but rather read 1 byte at a time for the
+	single command or compound command.  To get around shells
+	that don't do this, initially we pad with 1024 bytes of "####..."
+	which assumes that the shell is improperly gobbling only
+	1024 bytes at a time, then we exec the POSIX shell */
+
+
+	/* test for shell  with required POSIX feature
+		(echo "(sed -e s/a/d/);"; echo /)  | ksh -s 1>/dev/null 2>&1; echo $?
+	*/
+
+	prog = swlib_utilname_get();
+	if (strcmp(prog, "swremove") == 0) {
+		prog = "/";
+		swbis = "/";
+	} else {
+		swbis = "/_swbis";
+	}
+
+	strob_sprintf(buf, STROB_DO_APPEND,
+		"{\n"
+		"test -x /bin/bash && exec /bin/bash -s %s /_\"%s\" / / PSH=\"\\\"/bin/bash -s\\\"\"\n"
+		"## this makes sure ksh is the public domain ksh and not AT&T ksh\n"
+		"test -x /bin/ksh && (echo \"(sed -e s/a/d/);\"; echo / ) | /bin/ksh -s 1>/dev/null 2>&1 && "
+			"exec /bin/ksh -s %s /_\"%s\" / / PSH=\"\\\"/bin/ksh -s\\\"\"\n"
+		"test -x /usr/xpg4/bin/sh && exec /usr/xpg4/bin/sh -s %s /_\"%s\" / / PSH=\"\\\"/usr/xpg4/bin/sh -s\\\"\"\n"
+		"(echo \"(sed -e s/a/d/);\"; echo / ) | /bin/sh -s 1>/dev/null 2>&1 &&"
+			"exec /bin/sh -s %s /_\"%s\" / / PSH=\"\\\"/bin/sh -s\\\"\"\n"
+		"echo \"swbis: a suitable shell was not found\" 1>&2\n"
+		"exit 1\n"
+		"};\n\n",
+		swbis, prog,
+		swbis, prog,
+		swbis, prog,
+		swbis, prog
+	);
+
+	for (j=0; j<=14; j++) {
+		for (i=0; i<=10; i++)
+			strob_sprintf(buf, STROB_DO_APPEND, "########");
+		strob_sprintf(buf, STROB_DO_APPEND, "\n");
+	}
+	strob_sprintf(buf, STROB_DO_APPEND, "\n");
+}
