@@ -40,6 +40,7 @@
 #define SCAR_ARRAY_LEN 20
 
 #define SWP_RP_STATUS_NO_INTEGRITY	4
+#define SWP_RP_STATUS_NO_GNU_TAR	5
 
 #define SWLIST_PMODE_TEST1	"test1"  /* provides a scratch test area for distributions */
 #define SWLIST_PMODE_DEP1	"dep1"   /* dependecy check format that is used internally 
@@ -234,6 +235,7 @@ int swpl_session_lock(GB * G, SWICOL * swicol, char * target_path, int ofd, int
 char * swpl_looper_payload_routine(STROB * buf, int vlv, char * locked_region);
 int swpl_signature_policy_accept(GB * G, SWGPG_VALIDATE * w, int verbose_level, char * swspec_string);
 int swpl_remove_catalog_entry(GB * G, SWICOL * swicol, SWICAT_E * e, int ofd, char * iscpath);
+int swpl_restore_catalog_entry(GB * G, SWICOL * swicol, SWICAT_E * e, int ofd, char * iscpath);
 char * swpl_rename_suffix(STROB * buf);
 void swpl_tag_volatile_files(SWI_FILELIST * fl);
 int swpl_show_file_list(GB * G, SWI_FILELIST * fl);
@@ -245,6 +247,7 @@ swpl_make_verify_command_script_fragment(GB * G,
 int
 swpl_retrieve_files(GB * G, SWI * swi, SWICOL * swicol, SWICAT_E * e, SWI_FILELIST * file_list,
 	int ofd, int ifd, int archive_fd, char * pax_write_command_key, int ls_fd, int ls_verbose, FILE_DIGS * digs);
-int swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_attributes, int skip_volatile);
+int swpl_ls_fileset_from_iscat(SWI * swi, int ofd, int ls_verbose, int * available_attributes, int skip_volatile, int check_contents);
+void swpl_bashin_detect(STROB * buf);
 
 #endif
