@@ -152,7 +152,7 @@ fprintf(fp, "%s",
 "      --no-getconf      Don't use the getconf utility to set the PATH variable\n"
 "                        on the target host.\n"
 "      --use-getconf     This causes getconf to be used.\n"
-"      --shell-command={posix|sh|bash|ksh|dash}  bash is the default\n"
+"      --shell-command={detect|posix|sh|bash|ksh}  detect is the default\n"
 "      --ssh-options=options      single letter ssh options. e.g. --ssh-options=1\n"
 "   -A,--A,--enable-ssh-agent-forwarding\n"
 "      --a,--disable-ssh-agent-forwarding\n"
@@ -201,7 +201,7 @@ fprintf(fp, "%s",
 "               ~/.swbis/swbisdefaults\n"
 "        Command Line Syntax : -x option=option_argument\n"
 "  swremove.swbis_no_getconf		 = true # getconf is used\n"
-"  swremove.swbis_shell_command	         = bash\n"
+"  swremove.swbis_shell_command	         = detect\n"
 "  swremove.swbis_no_remote_kill	 = false\n"
 "  swremove.swbis_any_format             = false\n"
 "  swremove.swbis_forward_agent          = true\n"
@@ -864,7 +864,7 @@ main(int argc, char **argv)
 	eopt_verbose			= "1";
 	wopt_no_getconf			= CHARTRUE;
 	wopt_no_remote_kill		= CHARFALSE;
-	wopt_shell_command		= "bash";
+	wopt_shell_command		= "detect";
 	wopt_local_pax_write_command 	= "tar"; 
 	wopt_remote_pax_write_command 	= "tar";
 	wopt_local_pax_read_command 	= "tar";
@@ -1254,18 +1254,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 			break;
 		case 188:
 			wopt_shell_command = strdup(optarg);
-			if (
-				strcmp(wopt_shell_command, "bash") &&
-				strcmp(wopt_shell_command, "sh") &&
-				strcmp(wopt_shell_command, "ksh") &&
-				strcmp(wopt_shell_command, "ash") &&
-				strcmp(wopt_shell_command, "dash") &&
-				strcmp(wopt_shell_command, "posix")
-			) {
-				sw_e_msg(G, "illegal shell command: %s \n",
-					wopt_shell_command);
-				exit(1);
-			}
+			swc_check_shell_command_key(G, wopt_shell_command);
 			set_opta(opta, SW_E_swbis_shell_command, wopt_shell_command);
 			break;
 		case 195:
