@@ -270,6 +270,7 @@ char *
 looper_locked_region(STROB * buf, int vlv)
 {
 	strob_sprintf(buf, 0,
+		CSHID
 		"				# -----------------------------\n"
 		"				# The locked region begins here\n"
 		"				# -----------------------------\n"
@@ -298,6 +299,7 @@ looper_locked_region(STROB * buf, int vlv)
 		"				swexec_status=$sw_retval\n"
 		"				case $sw_retval in 0) %s ;; *) ;; esac\n" /* clear to send */
 	
+		CSHID
 		"				shls_bashin2 \"" SWBIS_TS_Catalog_unpack "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_Catalog_unpack  ";; *) shls_false_;; esac\n"
@@ -310,6 +312,7 @@ looper_locked_region(STROB * buf, int vlv)
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
 
+		CSHID
 		"				shls_bashin2 \"" SWBIS_TS_Catalog_dir_remove "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s " SWBIS_TS_Catalog_dir_remove ";; *) shls_false_;; esac\n"
@@ -354,9 +357,11 @@ swremove_write_source_copy_script2(GB * G,
 	char * opt_allow_no_lock;
 	char * ignore_scripts;
 	char * c_do_cleansh;
+	char * xx;
 	STROB * locked_region;
 	STROB * looper_routine;
 	STROB * buffer;
+	STROB * buffer_new;
 	STROB * shell_lib_buf;
 	STROB * is_archive_msg;
 	STROB * is_directory_msg;
@@ -373,6 +378,7 @@ swremove_write_source_copy_script2(GB * G,
 	basename = (char*)NULL;
 	dirname = (char*)NULL;
 	buffer = strob_open(100);
+	buffer_new = strob_open(100);
 	to_devnull = strob_open(100);
 	is_archive_msg = strob_open(100);
 	is_directory_msg = strob_open(100);
@@ -492,12 +498,18 @@ swremove_write_source_copy_script2(GB * G,
 	looper_locked_region(locked_region, vlv);
 	swpl_looper_payload_routine(looper_routine,  vlv, strob_str(locked_region));
 
+	strob_strcpy(buffer_new, "");
+	if (strcmp(get_opta(G->optaM, SW_E_swbis_shell_command), "detect") == 0) {
+		swpl_bashin_detect(buffer_new);
+	}
 
 	/* Now here is the script */
-	ret = swlib_writef(ofd, buffer, 	
+	strob_sprintf(buffer_new, STROB_DO_APPEND,
+	/* ret = swlib_writef(ofd, buffer,  */
 									/* ism_begin */
 		"trap '/bin/rm -f ${LOCKPATH}.lock; exit 1' 1 2 15\n"
 		"echo " SWBIS_TARGET_CTL_MSG_125 ": " KILL_PID "\n"
+		CSHID
 		"export LOCKENTRY\n"
 		"LOCKPATH=\"\"\n"
 		"%s\n"  			/* SEVENT: SW_SESSION_BEGINS */
@@ -536,7 +548,9 @@ swremove_write_source_copy_script2(GB * G,
 		"sw_retval=0\n"
 		"sb__delaytime=%d\n"
 		"export sh_dash_s\n"
-		"sh_dash_s=\"%s\"\n"
+		"d_sh_dash_s=\"%s\"\n"
+		"case \"$5\" in PSH=*) eval \"$5\";; *) unset PSH ;; esac\n"
+		"sh_dash_s=\"${PSH:=$d_sh_dash_s}\"\n"
 		"debug_task_shell=\"%s\"\n"
 		"swexec_status=0\n"
 		"export LOCKPATH\n"
@@ -547,6 +561,7 @@ swremove_write_source_copy_script2(GB * G,
 		/* Here, in this first if-then statement we will classify and
 		   type-test the target path */
 
+		CSHID
 		"case \"$do_cleansh\" in\n"
 		"	yes)\n"
 		"		%s\n"    /* Send is cleansh message */
@@ -607,6 +622,7 @@ swremove_write_source_copy_script2(GB * G,
 			notified via the "is_archive_msg" and "is_directory_msg"
 			and the $sw_targettype var has been set.  */
 
+		CSHID
 		"case \"$sw_targettype\" in\n"
 		"	\"regfile\")\n"
 				/* This is an error for swremove */
@@ -618,6 +634,7 @@ swremove_write_source_copy_script2(GB * G,
 		"		%s\n" /* SOURCE_ACCESS_BEGINS */
 		"		swexec_status=0\n"
 
+		CSHID
 		"shls_bashin2 \"" SWBIS_TS_uts "\"\n"
 		"sw_retval=$?\n"
 		"case $sw_retval in 0) $sh_dash_s " SWBIS_TS_uts ";; *) shls_false_;; esac\n"
@@ -629,6 +646,7 @@ swremove_write_source_copy_script2(GB * G,
 		"sw_retval=$?\n"
 		"swexec_status=$sw_retval\n"
 
+		CSHID
 		"shls_bashin2 \"" SWBIS_TS_Get_iscs_listing "\"\n"
 		"sw_retval=$?\n"
 		"case $sw_retval in 0) $sh_dash_s " SWBIS_TS_Get_iscs_listing ";; *) shls_false_;; esac\n"
@@ -658,6 +676,7 @@ swremove_write_source_copy_script2(GB * G,
 		"		;;\n"
 		"esac  # case sw_targettype\n"
 
+		CSHID
 		"if test \"$sw_retval\" != \"0\"; then\n"
 		"	 sb__delaytime=0;\n"
 		"fi\n"
@@ -689,7 +708,7 @@ swremove_write_source_copy_script2(GB * G,
 /*_% */		basename,
 /*_% */		targetpath,
 /*_% */		delaytime,
-/*_% */		G->g_sh_dash_s, 
+/*_% */		swc_get_default_sh_dash_s(G),
 /*_% */		debug_task_shell,
 /*_% */		strob_str(looper_routine),
 /*_% */		strob_str(is_task_msg),
@@ -711,15 +730,22 @@ swremove_write_source_copy_script2(GB * G,
 /*_% */		swicol_subshell_marks(subsh2, "install_target", 'R', nhops, vlv)
 		);
 
+	xx = strob_str(buffer_new);
+	ret = atomicio((ssize_t (*)(int, void *, size_t))write, ofd, xx, strlen(xx));
+	if (ret != (int)strlen(xx)) {
+		return 1;
+	}
+
 	free(basename);
 	free(dirname);
 	strob_close(tmp);
 	if (G->g_source_script_name) {
-		swlib_tee_to_file(G->g_source_script_name, -1, strob_str(buffer), -1, 0);
+		swlib_tee_to_file(G->g_source_script_name, -1, xx, -1, 0);
 	}
 
 	strob_close(locked_region);
 	strob_close(looper_routine);
+	strob_close(buffer_new);
 	strob_close(buffer);
 	strob_close(is_archive_msg);
 	strob_close(is_directory_msg);
