@@ -163,109 +163,12 @@ show_file_list(GB * G, SWI_FILELIST * fl)
 	return 0;
 }
 
-static /* same as in swproglib.c */
-int
-make_verify_command_script_fragment(GB * G, STROB * scriptbuf, SWI_FILELIST * fl, char * pax_write_command_key)
-{
-	int ret;
-	STROB * stop_word;
-	char * name;
-	int ix;
-	int check_volatile;
-	
-	/* Tag every file in a volatile directory as being volatile itself
-	   FIXME ??? is this the correct thing to do */
-
-	check_volatile = swextopt_is_option_true(SW_E_check_volatile, G->optaM);
-
-	E_DEBUG("");
-	swpl_tag_volatile_files(fl);
-	
-	stop_word = strob_open(32);
-
-	ret = determine_here_document_stop_word(fl, stop_word);
-	if (ret != 0) {
-		/* this really should never happen */
-		exit(12);
-	}
-
-	E_DEBUG("");
-
-	/* sort the files in forward direction */
-	swi_fl_qsort_forward(fl);
-
-	strob_sprintf(scriptbuf, STROB_DO_APPEND, "rm_retval=0\n");
-
-	/* Construct the here document of filenames that feed
-	   GNU tar's stdin to make an archive */
-	/* Something like this:
-		(
-		cat <<'/tmp/End'
-		a
-		c
-		/tmp/End
-		) | tar  cf - --no-recursion --files-from=-
-	*/
-
-	/* Start of hear document */
-	strob_sprintf(scriptbuf, STROB_DO_APPEND,
-		"(\n"
-		"cat <<'%s'\n",
-		strob_str(stop_word));
-
-	/* priint each file */
-	E_DEBUG("");
-	ix = 0;
-	while ((name=swi_fl_get_path(fl, ix)) != NULL) {
-		if (G->devel_verboseM) {
-			SWLIB_INFO3("INFO; file=[%s] type=[%c]", name, (int)swi_fl_get_type(fl, ix));
-		}
-
-		/* check_volatile */
-		if (check_volatile == 0 && swi_fl_is_volatile(fl, ix)) {
-			/* skip */
-			;
-		} else {
-			strob_sprintf(scriptbuf, STROB_DO_APPEND, "%s\n", name);
-		}
-		ix++;
-	}
-
-	/* End of here document */
-	E_DEBUG("");
-	strob_sprintf(scriptbuf, STROB_DO_APPEND,
-		"%s\n"
-		") |",
-		strob_str(stop_word));
-
-
-	/* Sanity check on pax_write_command_key */
-
-	if (strcmp(pax_write_command_key, "tar") == 0) {
-		;
-	} else if (strcmp(pax_write_command_key, "gtar") == 0) {
-		;
-	} else {
-		/* This is an error, should write a warning  */
-		pax_write_command_key = "tar";
-	}
-	
-	/* write the tar writing command */
-
-	E_DEBUG("");
-	strob_sprintf(scriptbuf, STROB_DO_APPEND, " %s vcf - -b 1 --no-recursion --files-from=- \n", pax_write_command_key);
-	strob_sprintf(scriptbuf, STROB_DO_APPEND, "retrieve_retval=$?\n");
-
-	E_DEBUG("");
-	strob_close(stop_word);
-	return 0;
-}
-
 static
 char *
 looper_locked_region(STROB * buf, int vlv)
 {
 	strob_sprintf(buf, 0,
+		CSHID
 		"				# -----------------------------\n"
 		"				# The locked region begins here\n"
 		"				# -----------------------------\n"
@@ -294,6 +197,7 @@ looper_locked_region(STROB * buf, int vlv)
 		"				swexec_status=$sw_retval\n"
 		"				case $sw_retval in 0) %s ;; *) ;; esac\n" /* clear to send */
 	
+		CSHID
 		"				shls_bashin2 \"" SWBIS_TS_Catalog_unpack "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s;; *) shls_false_;; esac\n"
@@ -306,6 +210,12 @@ looper_locked_region(STROB * buf, int vlv)
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
 
+		"				case $swexec_status in\n"
+		"			"	xSTR(SWP_RP_STATUS_NO_GNU_TAR) ") swexec_status=0\n"
+		"				;;\n"
+		"				esac\n"
+
+		CSHID
 		"				shls_bashin2 \"" SWBIS_TS_Catalog_dir_remove "\"\n"
 		"				sw_retval=$?\n"
 		"				case $sw_retval in 0) $sh_dash_s;; *) shls_false_;; esac\n"
@@ -318,6 +228,7 @@ looper_locked_region(STROB * buf, int vlv)
 		"				sw_retval=$?\n"
 		"				swexec_status=$sw_retval\n"
 
+		CSHID
 		"				%s\n" /* SW_SELECTION_EXECUTION_ENDS */
 		,
 /*_% */		TEVENT(2, vlv,  SW_SELECTION_EXECUTION_BEGINS, "$swspec_string"),
@@ -371,7 +282,7 @@ compare_names(char * mname, char * sname)
 
 static
 int
-compare_diff(GB * G, char * host, char * swspec_string, char * target_path, STRAR * meta, STRAR * sys)
+compare_diff(GB * G, char * host, char * swspec_string, char * target_path, STRAR * meta, STRAR * sys, int do_show)
 {
 	int retval;
 	char * m_s;
@@ -413,6 +324,12 @@ compare_diff(GB * G, char * host, char * swspec_string, char * target_path, STRA
 			im++;			
 			is++;			
 			E_DEBUG("match");
+			if (do_show) {
+				strob_sprintf(diff, 0,
+					"= %s\n",
+		 			s_s);
+				fprintf(stdout, "%s", strob_str(diff));
+			}
 			;
 		} else if (m_s && s_s && strcmp(m_s, s_s) != 0) {
 			/* either changed of deleted file */
@@ -494,25 +411,38 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 	int check_volatile;
 	int check_mtime;
 	int check_permissions;
+	int check_contents;
+	int compare_status_is_set;
 
+	compare_status_is_set = 0;
 	*compare_status = -1;
 	meta_fd = swlib_open_memfd();
 	sys_fd = swlib_open_memfd();
 	nullfd = swlib_open_nullfd();
 	tmp = strob_open(100);
 
+	available_attributes = 0;
 	/* list the catalog metadata */
 
 	check_volatile = swextopt_is_option_true(SW_E_check_volatile, G->optaM);
 	check_mtime = swextopt_is_option_true(SW_E_swbis_check_mtime, G->optaM);
 	check_permissions = swextopt_is_option_true(SW_E_check_permissions, G->optaM);
+	check_contents = swextopt_is_option_true(SW_E_check_contents, G->optaM);
+
+	if (check_contents == 0)
+		check_mtime = 0; /* per spec */
 	
 	ls_attributes = LS_LIST_VERBOSE_COMP2;
+	ls_attributes &= ~(LS_LIST_VERBOSE_WITH_MD5);
+	ls_attributes &= ~(LS_LIST_VERBOSE_WITH_SHA1);
+	ls_attributes &= ~(LS_LIST_VERBOSE_WITH_SHA512);
+
 	if (check_mtime == 0) {
 		ls_attributes &= ~(LS_LIST_VERBOSE_WITH_REG_DATES|LS_LIST_VERBOSE_WITH_ALL_DATES);
 	}
-
-	swpl_ls_fileset_from_iscat(swi, meta_fd, ls_attributes, &available_attributes, !check_volatile);
+	
+	swpl_ls_fileset_from_iscat(swi, meta_fd, ls_attributes,
+			&available_attributes, !check_volatile, check_contents);
 
 	/* Now use the value of &available_attributes to set the
 	   LS_LIST_VERBOSE_* controls.  This is required, for example,
@@ -523,7 +453,10 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 	   formed for uid/uname and digests that are in the meta-data.
 	   This is represented in (int)available_attributes */
 
-	available_attributes |= LS_LIST_VERBOSE_ALTER_FORM|LS_LIST_VERBOSE_NORMAL|LS_LIST_VERBOSE_STRIPSLASH|LS_LIST_VERBOSE_WITH_REG_DATES;
+	available_attributes |= LS_LIST_VERBOSE_ALTER_FORM|LS_LIST_VERBOSE_NORMAL |
+					/* LS_LIST_VERBOSE_WITH_SIZE | */
+					LS_LIST_VERBOSE_STRIPSLASH |
+					LS_LIST_VERBOSE_WITH_REG_DATES;
 	available_attributes &= ~LS_LIST_VERBOSE_OFF;
 	if (check_mtime == 0) {
 		available_attributes &= ~(LS_LIST_VERBOSE_WITH_REG_DATES|LS_LIST_VERBOSE_WITH_ALL_DATES);
@@ -535,12 +468,41 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 	file_hdr = ahs_vfile_hdr(xformat_ahs_object(xformat));
 	taru_init_header_digs(file_hdr);
 	digs = file_hdr->digsM;
-	if (available_attributes & LS_LIST_VERBOSE_WITH_MD5)
-		digs->do_md5 = DIGS_ENABLE_ON;
-	if (available_attributes & LS_LIST_VERBOSE_WITH_SHA1)
-		digs->do_sha1 = DIGS_ENABLE_ON;
-	if (available_attributes & LS_LIST_VERBOSE_WITH_SHA512)
-		digs->do_sha512 = DIGS_ENABLE_ON;
+
+	if (check_contents == 0) {
+		digs->do_md5 = DIGS_ENABLE_OFF;
+		digs->do_sha1 = DIGS_ENABLE_OFF;
+		digs->do_sha512 = DIGS_ENABLE_OFF;
+		available_attributes &= ~(LS_LIST_VERBOSE_WITH_MD5|LS_LIST_VERBOSE_WITH_SHA1|LS_LIST_VERBOSE_WITH_SHA512);
+		available_attributes &= ~(LS_LIST_VERBOSE_WITH_SIZE|LS_LIST_VERBOSE_WITH_ALL_DATES);
+	} else {
+		available_attributes |= (LS_LIST_VERBOSE_WITH_SIZE);
+		digs->do_md5 = DIGS_ENABLE_OFF;
+		digs->do_sha1 = DIGS_ENABLE_OFF;
+		digs->do_sha512 = DIGS_ENABLE_OFF;
+		if (available_attributes & LS_LIST_VERBOSE_WITH_MD5) {
+			digs->do_md5 = DIGS_ENABLE_ON;
+		}
+		if (available_attributes & LS_LIST_VERBOSE_WITH_SHA1) {
+			digs->do_sha1 = DIGS_ENABLE_ON;
+		}
+		if (available_attributes & LS_LIST_VERBOSE_WITH_SHA512) {
+			digs->do_sha512 = DIGS_ENABLE_ON;
+		}
+
+		if (
+			(available_attributes & LS_LIST_VERBOSE_WITH_MD5) == 0 &&
+			(available_attributes & LS_LIST_VERBOSE_WITH_SHA1) == 0 &&
+			(available_attributes & LS_LIST_VERBOSE_WITH_SHA512) == 0
+		) {
+			/* The the package contains no digests, verification
+			   must therefore fail */
+			compare_status_is_set = 1;
+			*compare_status = 1;
+			fprintf(stderr, "%s: error: the package does not contain file digests\n", swlib_utilname_get());
+		}
+
+	}
 
 	E_DEBUG("");
 	uxfio_lseek(archive_fd, 0, SEEK_SET);
@@ -577,10 +539,18 @@ compare_fileset(GB * G, char * swspec_string, char * target_path, SWUTS * uts, S
 
 	/* Now compare these line by line */
 
-	*compare_status = compare_diff(G, G->g_target_terminal_host, swspec_string, target_path, meta_str, sys_str);
+	ret = compare_diff(G, G->g_target_terminal_host, swspec_string,
+				target_path, meta_str, sys_str, G->g_do_show_linesM);
 
+	if (compare_status_is_set == 0) {
+		compare_status_is_set = 1;
+		*compare_status = ret;
+	}
+
+	/*
 	print_lines("/tmp/meta.txt", "", meta_str);
 	print_lines("/tmp/sys.txt", "", sys_str);
+	*/
 
 	strar_close(meta_str);
 	strar_close(sys_str);
@@ -613,9 +583,11 @@ swverify_write_source_copy_script2(GB * G,
 	char * opt_force_lock;
 	char * opt_allow_no_lock;
 	char * ignore_scripts;
+	char * xx;
 	STROB * locked_region;
 	STROB * looper_routine;
 	STROB * buffer;
+	STROB * buffer_new;
 	STROB * shell_lib_buf;
 	STROB * is_archive_msg;
 	STROB * is_directory_msg;
@@ -631,6 +603,7 @@ swverify_write_source_copy_script2(GB * G,
 	basename = (char*)NULL;
 	dirname = (char*)NULL;
 	buffer = strob_open(100);
+	buffer_new = strob_open(100);
 	to_devnull = strob_open(100);
 	is_archive_msg = strob_open(100);
 	is_directory_msg = strob_open(100);
@@ -740,12 +713,19 @@ swverify_write_source_copy_script2(GB * G,
 	looper_locked_region(locked_region, vlv);
 	swpl_looper_payload_routine(looper_routine,  vlv, strob_str(locked_region));
 
+	strob_strcpy(buffer_new, "");
+	if (strcmp(get_opta(G->optaM, SW_E_swbis_shell_command), "detect") == 0) {
+		swpl_bashin_detect(buffer_new);
+	}
+
 	/* Now here is the script */
-	ret = swlib_writef(ofd, buffer, 	
+	/* ret = swlib_writef(ofd, buffer,  */
+	strob_sprintf(buffer_new, STROB_DO_APPEND,
 									/* ism_begin */
 		"trap '/bin/rm -f ${LOCKPATH}.lock; exit 1' 1 2 15\n"
 		"echo " SWBIS_TARGET_CTL_MSG_125 ": " KILL_PID "\n"
 		"export LOCKENTRY\n"
+		CSHID
 		"LOCKPATH=\"\"\n"
 		"%s\n"  			/* SEVENT: SW_SESSION_BEGINS */
 		"%s" 				/* swicol_subshell_marks */
@@ -779,7 +759,9 @@ swverify_write_source_copy_script2(GB * G,
 		"sw_retval=0\n"
 		"sb__delaytime=%d\n"
 		"export sh_dash_s\n"
-		"sh_dash_s=\"%s\"\n"
+		"d_sh_dash_s=\"%s\"\n"
+		"case \"$5\" in PSH=*) eval \"$5\";; *) unset PSH ;; esac\n"
+		"sh_dash_s=\"${PSH:=$d_sh_dash_s}\"\n"
 		"debug_task_shell=\"%s\"\n"
 		"swexec_status=0\n"
 		"export LOCKPATH\n"
@@ -790,6 +772,7 @@ swverify_write_source_copy_script2(GB * G,
 		/* Here, in this first if-then statement we will classify and
 		   type-test the target path */
 									/* ism_test_begin */
+		CSHID
 		"if test -e \"$dirname\"; then\n"
 		"	cd \"$dirname\"\n"
 		"	sw_retval=$?\n"
@@ -836,6 +819,7 @@ swverify_write_source_copy_script2(GB * G,
 			notified via the "is_archive_msg" and "is_directory_msg"
 			and the $sw_targettype var has been set.  */
 
+		CSHID
 		"case \"$sw_targettype\" in\n"
 		"	\"regfile\")\n"
 				/* This is an error for swverify */
@@ -847,6 +831,7 @@ swverify_write_source_copy_script2(GB * G,
 		"		%s\n"  
 		"		swexec_status=0\n"
 
+		CSHID
 		"shls_bashin2 \"" SWBIS_TS_uts "\"\n"
 		"sw_retval=$?\n"
 		"case $sw_retval in 0) $sh_dash_s;; *) shls_false_;; esac\n"
@@ -887,6 +872,7 @@ swverify_write_source_copy_script2(GB * G,
 		"		;;\n"
 		"esac\n"
 
+		CSHID
 		"if test \"$sw_retval\" != \"0\"; then\n"
 		"	 sb__delaytime=0;\n"
 		"fi\n"
@@ -915,7 +901,7 @@ swverify_write_source_copy_script2(GB * G,
 /*_% */		basename,
 /*_% */		targetpath,
 /*_% */		delaytime,
-/*_% */		G->g_sh_dash_s, 
+/*_% */		swc_get_default_sh_dash_s(G),
 /*_% */		debug_task_shell,
 /*_% */		strob_str(looper_routine),
 /*_% */		strob_str(is_directory_msg),
@@ -934,11 +920,17 @@ swverify_write_source_copy_script2(GB * G,
 /*_% */		swicol_subshell_marks(subsh2, "install_target", 'R', nhops, vlv)
 		);
 
+	xx = strob_str(buffer_new);
+	ret = atomicio((ssize_t (*)(int, void *, size_t))write, ofd, xx, strlen(xx));
+	if (ret != (int)strlen(xx)) {
+		return 1;
+	}
+
 	free(basename);
 	free(dirname);
 	strob_close(tmp);
 	if (G->g_source_script_name) {
-		swlib_tee_to_file(G->g_source_script_name, -1, strob_str(buffer), -1, 0);
+		swlib_tee_to_file(G->g_source_script_name, -1, xx, -1, 0);
 	}
 
 	strob_close(locked_region);
@@ -1329,6 +1321,9 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 				looper_abort(G, swicol, ofd);
 				goto error_out;
 			} else if (ret > 0) {
+				if (ret == SWP_RP_STATUS_NO_GNU_TAR) {
+					sw_e_msg(G, "GNU tar or pax is required for this operation but appears missing.\n");
+				}
 				rstatus = 9;
 			}
 		}
@@ -1339,10 +1334,15 @@ swverify_looper_sr_payload(GB * G, char * target_path, SWICOL * swicol,
 	   files is in (int)archive_fd */
 	E_DEBUG("");
 
-	if (archive_fd > 0) {
-		/* compare the archive to the catalog */
-		ret = compare_fileset(G, strob_str(swspec_string), target_path, uts, swi, archive_fd, compare_status);
-		uxfio_close(archive_fd);
+	if (rstatus) {
+		*compare_status = 1;
+		ret = 0;
+	} else {
+		if (archive_fd > 0) {
+			/* compare the archive to the catalog */
+			ret = compare_fileset(G, strob_str(swspec_string), target_path, uts, swi, archive_fd, compare_status);
+			uxfio_close(archive_fd);
+		}
 	}
 	archive_fd = -1;
 
