@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,18 +17,23 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef swexfileset_hxx
 #define swexfileset_hxx
 
-#include "swuser_assert_config.h"
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include "swuser_assert_config.h"
 #include "swmetadata.h"
 #include "swexstruct.h"
 #include "swexstruct_i.h"
@@ -45,8 +50,12 @@ extern "C" {
 #include "swvarfs.h"
 #include "swheaderline.h"
 #include "taru.h"
+#include "strtoint.h"
 }
 
+// #define SWEXFILESETNEEDDEBUG 1
+// #undef SWEXFILESETNEEDDEBUG
+
 #ifdef SWEXFILESETNEEDDEBUG
 #define SWEXFILESET_DEBUG(format) SWBISERROR("SWEXFILESET DEBUG: ", format)
 #define SWEXFILESET_DEBUG2(format, arg) SWBISERROR2("SWEXFILESET DEBUG: ", format, arg)
@@ -61,7 +70,7 @@ class swExFileset: public swObjFiles_i
 {
 	swPtrList<swPackageFile> * distributionFilesM;
 	STROB * sbufM;
-	unsigned long int sizeM;
+	uintmax_t sizeM;
       public:
 	
 	swExFileset(char * control_dir): swObjFiles_i(control_dir) {
@@ -259,13 +268,15 @@ class swExFileset: public swObjFiles_i
 
 	void generateFileSetSize(void) {
 		swINFO * info = getInfo();
-		unsigned long int bytesize = 0;
+		uintmax_t bytesize = 0;
 		swDefinition * inext;
 		swMetaData * att;
 		swDefinition *swdef = getReferer();
-		char asize[24];
+		char * asize;
+		STROB * uimaxbuf;
 
 		SWEXFILESET_DEBUG("entering");
+		uimaxbuf = strob_open(24);
 		if (info == NULL) return;
 		inext = info->swdeffile_linki_get_head();
 
@@ -273,16 +284,16 @@ class swExFileset: public swObjFiles_i
 			att = inext->get_next_node();  // The first attribute.
 			while (att) {
 				if ( ::strcmp(att->get_keyword(), SW_A_size) == 0) {
-					bytesize += (unsigned int)(::atoi(att->get_value(NULL)));
+					bytesize += ::strtoumax(att->get_value(NULL), NULL, 10);
 				}
 				att = att->get_next_node();
 			}
 			inext = inext->get_next();
 		}
 		sizeM = bytesize;
-		snprintf(asize, sizeof(asize) -1, "%lu", sizeM); 
-		asize[sizeof(asize) - 1] = '\0';
+		asize = swlib_umaxtostr(sizeM, uimaxbuf);
 		swdef->add(SW_A_size, asize);
+		strob_close(uimaxbuf);
 		SWEXFILESET_DEBUG("leaving");
 	}
 	
@@ -336,7 +347,7 @@ class swExFileset: public swObjFiles_i
 		char * value;
 		int ofd;
 		int preview_fd;
-		int ret;
+		intmax_t ret;
 		int skip;
 		int do_adjunct = 0;
 		swPackageFile * archiver;
@@ -485,14 +496,35 @@ class swExFileset: public swObjFiles_i
 					stats_swdef->write_fd(2);
 					SWEXFILESET_DEBUG2("<<<<< writing path=[%s]", path);
 				#endif
-				ret = archiver->xFormat_write_file(stats_swdef, strob_str(tmp));
+				/* Handle a root directory in the package as
+				   a special case */
+				if (strcmp("/", path) == 0) {
+					if (
+						strcmp(strob_str(tmp), ".") == 0 ||
+						strob_strlen(tmp) < 3 /* arbitrary sanity limit */
+					) {
+						;
+						//fprintf(stderr,
+                                        	//"%s: Warning: root directory member processing exception\n",
+	                                        //swlib_utilname_get());
+					} else {
+						swlib_unix_dircat(tmp, "./");
+					}
+				}
+				SWEXFILESET_DEBUG2("<<<<< writing [%s]", tmp->str_);
+				/* Final sanity check on the archive path */
+				if (strstr(strob_str(tmp), "../")) {
+					ret = -1;
+				} else {
+					ret = archiver->xFormat_write_file(stats_swdef, strob_str(tmp));
+				}
 			} 
 			skip = 0;	
 			
 			if (ret < 0) {
 				fprintf(stderr,
-					"%s: error writing archive member for ( %s ) ret=%d\n",
-					swlib_utilname_get(), strob_str(tmp), ret);
+					"%s: error writing archive member for ( %s ) ret=%s\n",
+					swlib_utilname_get(), strob_str(tmp), swlib_imaxtostr(ret, NULL));
 				setErrorCode(12303, NULL);
 				return;
 			}
