@@ -502,7 +502,14 @@ class swObjFiles_i:  public swExStruct_i
 
 			// fprintf(stderr, "eraseme swpackage: checking for duplicate here [%s]\n", path);
 			// swmd->write_fd(2);  // Heres howto write the definition to STDERR_FILENO
-			did_find_duplicate = check_for_storage_section_duplicates(filedeflist, pathlist, path, basenames);
+
+			if (
+				get_opta_array() == NULL ||
+				::swextopt_is_option_true(SW_E_swbis_check_duplicates, get_opta_array())
+			)
+				did_find_duplicate = check_for_storage_section_duplicates(filedeflist, pathlist, path, basenames);
+			else
+				did_find_duplicate = 0;
 
 			swmd_def = static_cast<swDefinition*>(swmd);
 			//
