@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include "swuser_config.h"
@@ -125,7 +125,8 @@ int swPackageFile::initialize_file_stats_array(swDefinition * swdef,
 	return 0;
 }
 
-int swPackageFile::apply_file_stats(swDefinition * swdef, SWVARFS * swvarfs, char *source, int cksumflags ) {
+int
+swPackageFile::apply_file_stats(swDefinition * swdef, SWVARFS * swvarfs, char *source, int cksumflags ) {
 	int ret;
 	struct stat st;
 	char fileArray			[lastE+1][file_permsLength];
@@ -189,9 +190,13 @@ int swPackageFile::apply_file_stats(swDefinition * swdef, SWVARFS * swvarfs, cha
 	
 	ret = swMetaData::swmetadata_decode_file_stats(swvarfs1, source, source,
 				&st, fileArray, fileArrayWasSet, cksumflags);
-	if (ret) {
+	if (ret < 0) {
 		SWPACKAGEFILE_DEBUG("");
+		// error
 		return ret;
+	} else if (ret > 0) {
+		// ignore file
+		ret = 0;
 	}
 
 	for (i=0; i<lastE; i++) {
