@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
@@ -71,7 +71,7 @@ class swStruct_i: public swStruct
      swStruct * get_pointer_from_index (int i){ return NULL;}
 
      int write_node(int uxfio_fd);
-     int write(int uxfio_fd);
+     int iwrite(int uxfio_fd);
      int write_debug(int uxfio_fd);
      void  set_iterator(swStructIter * clswit);
      swStructIter *  get_iterator(void);
