@@ -107,6 +107,39 @@
 #define UGSWIDTH 18
 static int ugswidth = UGSWIDTH;
 
+static int encoding_flag;
+
+void
+ls_list_set_encoding_flag(int flag)
+{
+	encoding_flag = flag;
+}
+
+int
+ls_list_get_encoding_flag(void)
+{
+	if (encoding_flag == 0) {
+		/* default encoding if not set */
+		return VIS_NONE;
+	}
+	return encoding_flag;
+}
+
+int
+ls_list_set_encoding_by_lang(void)
+{
+	int flag;
+	char * e;
+
+	e = getenv("LANG");
+	if (e && strcmp(e, "C") == 0) {
+		flag = VIS_OCTAL;
+	} else {
+		flag = VIS_NONE;
+	}
+	ls_list_set_encoding_flag(flag);
+	return flag;
+}
 
 static
 char *
@@ -177,7 +210,7 @@ safe_print (char *str, FILE * fp)
     {
       for (cp = str; *cp; cp++)
 	{
-	  (void) vis (visbuf, cp[0], VIS_CSTYLE, cp[1]);
+	  (void) vis (visbuf, cp[0], ls_list_get_encoding_flag(), cp[1]);
 	  (void) fputs (visbuf, fp);
 	}
     }
@@ -187,9 +220,8 @@ safe_print (char *str, FILE * fp)
     }
 }
 
-static
 void
-safe_print_to_strob(char *str, STROB * fp)
+ls_list_safe_print_to_strob(char *str, STROB * fp)
 {
   char visbuf[5];
   char *cp;
@@ -198,7 +230,9 @@ safe_print_to_strob(char *str, STROB * fp)
     {
       for (cp = str; *cp; cp++)
 	{
-	  (void) vis (visbuf, cp[0], VIS_CSTYLE, cp[1]);
+	  /* visbuf[0] = *cp; visbuf[1] = '\0'; */
+
+	  (void) vis (visbuf, cp[0], ls_list_get_encoding_flag(), cp[1]);
 	  strob_strcat(fp, visbuf);
 	}
     }
@@ -311,23 +345,38 @@ print_linkname(STROB * fp, int type, char * ln_name)
 	if ((type == PAX_HLK) || (type == PAX_HRG)) {
 		/* strob_strcat(fp, " == "); */
 		strob_strcat(fp, " link to ");
-		safe_print_to_strob(ln_name, fp);
+		ls_list_safe_print_to_strob(ln_name, fp);
 	} else {
 		if (type == PAX_SLK) {
 			strob_strcat(fp, " -> ");
-			safe_print_to_strob(ln_name, fp);
+			ls_list_safe_print_to_strob(ln_name, fp);
     		}
 	}
 }
 
+static
+void
+print_dig(STROB * fp, struct new_cpio_header * file_hdr, int do_dig, char * key, char * val)
+{
+	strob_strcat(fp, key);
+	if (file_hdr->digsM->do_poisonM && strlen(val) == 0 ) {
+		strob_sprintf(fp, 1, "<not available>");
+	} else {
+		strob_sprintf(fp, 1, "%s", val);
+	}
+}
+
 static
 void
 print_sha1(STROB * fp, struct new_cpio_header * file_hdr)
 {
-	if (file_hdr->digsM && file_hdr->digsM->do_sha1) {
+	print_dig(fp, file_hdr, file_hdr->digsM->do_sha1, "SHA1=", file_hdr->digsM->sha1);
+	/*
+	if (file_hdr->digsM && file_hdr->digsM->do_sha1 == DIGS_ENABLE_ON) {
 		strob_strcat(fp, "SHA1=");
 		strob_sprintf(fp, 1, "%s", file_hdr->digsM->sha1);
 	}
+	*/
 }
 
 
@@ -335,10 +384,13 @@ static
 void
 print_md5(STROB * fp, struct new_cpio_header * file_hdr)
 {
-	if (file_hdr->digsM && file_hdr->digsM->do_md5) {
+	print_dig(fp, file_hdr, file_hdr->digsM->do_md5, "MD5=", file_hdr->digsM->md5);
+	/*
+	if (file_hdr->digsM && file_hdr->digsM->do_md5 == DIGS_ENABLE_ON) {
 		strob_strcat(fp, "MD5=");
 		strob_sprintf(fp, 1, "%s", file_hdr->digsM->md5);
 	}
+	*/
 }
 
 static
@@ -358,13 +410,15 @@ alt_ls_list_to_string(char * fp_name, char * ln_name, struct new_cpio_header * f
 
 	mtime = (time_t)(file_hdr->c_mtime);
 	strob_strcpy(fp, "");
-	safe_print_to_strob(name, fp);
+	ls_list_safe_print_to_strob(name, fp);
 	strob_strcat(fp, " " LS_LIST_LINKNAME_MARK);
 	print_linkname(fp, type, ln_name);
 	strob_strcat(fp, "]");
 	
+	/* filesize */
 	strob_strcat(fp, " [");
-	print_size_rdev(fp, file_hdr, type);
+	if (vflag & LS_LIST_VERBOSE_WITH_SIZE)
+		print_size_rdev(fp, file_hdr, type);
 	strob_strcat(fp, "]");
 
 	strob_strcat(fp, " ");
@@ -376,12 +430,14 @@ alt_ls_list_to_string(char * fp_name, char * ln_name, struct new_cpio_header * f
 
 	strob_strcat(fp, " [");
 	if (type == REGTYPE)
-		print_md5(fp, file_hdr);
+		if (vflag & LS_LIST_VERBOSE_WITH_MD5)
+			print_md5(fp, file_hdr);
 	strob_strcat(fp, "]");
 	
 	strob_strcat(fp, " [");
 	if (type == REGTYPE)
-		print_sha1(fp, file_hdr);
+		if (vflag & LS_LIST_VERBOSE_WITH_SHA1)
+			print_sha1(fp, file_hdr);
 	strob_strcat(fp, "]");
 		
 	if (type == DIRTYPE)
@@ -445,7 +501,7 @@ ls_list_to_string(char * fp_name, char * ln_name, struct new_cpio_header * file_
 	strob_strcat(fp, " ");
 
 	/* Print the name */
-	safe_print_to_strob(name, fp);
+	ls_list_safe_print_to_strob(name, fp);
 
 	/* Print the linkname for symlinks and hard links */
 	print_linkname(fp, type, ln_name);
