@@ -713,6 +713,28 @@ parse_options_files(struct extendedOptions * opta, char * option_files,
 }
 
 
+static
+int
+has_file_part(char * soc)
+{
+	if (!soc || strlen(soc) == 0) return 0;
+	if (strchr(soc, ':')) {
+		/* is it the last one */
+		if (*(soc+(strlen(soc)-1)) == ':') {
+			/* yes, the last one indeed 
+			   i.e. form    host:host: */
+
+			/* no file part */
+			return 0;
+		} else {
+			/* has file part */
+			return 1;
+		}
+	}
+	return 0;
+}
+
+
 static
 int
 is_host(char * soc)
@@ -888,6 +910,7 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 	else if (
 		!is_host(directory) &&
 		is_host(soc_spec) &&
+		has_file_part(soc_spec) == 0 &&
 		1
 	) {
 		/*
@@ -905,6 +928,12 @@ swextopt_combine_directory(STROB * result, char * soc_spec, char * directory)
 		 */
 		E_DEBUG("case");
 		combine1(result, soc_spec, directory);
+	} else if (
+		is_host(soc_spec) &&
+		has_file_part(soc_spec) &&
+		1
+	) {
+		; /* do nothing */
 	} else {
 		E_DEBUG("null case");
 		retval = -1;
