@@ -336,13 +336,13 @@ swlib_i_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int adjunc
 	
 	oldt = newt = time(NULL);
 
-	if (digs && digs->do_md5)
+	if (digs && digs->do_md5 == DIGS_ENABLE_ON)
 		swlib_md5_from_memblocks(&md5ctx, digs->md5, (unsigned char*)NULL, -1);
 
-	if (digs && digs->do_sha1)
+	if (digs && digs->do_sha1 == DIGS_ENABLE_ON)
 		sha_block((void*)sha1ctx, res_sha1, NULL, -1);
 
-	if (digs && digs->do_sha512)
+	if (digs && digs->do_sha512 == DIGS_ENABLE_ON)
 		sha512_block((void*)sha512ctx, res_sha512, NULL, -1);
 
 	do {
@@ -354,7 +354,7 @@ swlib_i_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int adjunc
 		n = (*f)(ifd, buf, readamount);
 		if (n < 0) return -1;
 		if (n) {
-			if (digs && digs->do_md5) {
+			if (digs && digs->do_md5 == DIGS_ENABLE_ON) {
 				p5 = (char*)buf;
 				am = n;
 				while(am > 0) {
@@ -366,10 +366,10 @@ swlib_i_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int adjunc
 				}
 			}
 	
-			if (digs && digs->do_sha1)
+			if (digs && digs->do_sha1 == DIGS_ENABLE_ON)
 				sha_block((void*)sha1ctx, res_sha1, buf, n);
 	
-			if (digs && digs->do_sha512)
+			if (digs && digs->do_sha512 == DIGS_ENABLE_ON)
 				sha512_block((void*)sha512ctx, res_sha512, buf, n);
 
 			if (ofd > 0) {
@@ -418,15 +418,15 @@ swlib_i_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int adjunc
 	} while (retval < count || count < 0);
 	if (n < 0) return -1;
 
-	if (digs && digs->do_md5) {
+	if (digs && digs->do_md5 == DIGS_ENABLE_ON) {
 		swlib_md5_from_memblocks(&md5ctx, digs->md5, (unsigned char*)NULL, 0);
 		digs->md5[32] = '\0';
 	}
 
-	if (digs && digs->do_sha1)
+	if (digs && digs->do_sha1 == DIGS_ENABLE_ON)
 		sha_block((void*)sha1ctx, res_sha1, (unsigned char*)NULL, 0); 
 
-	if (digs && digs->do_sha512)
+	if (digs && digs->do_sha512 == DIGS_ENABLE_ON)
 		sha512_block((void*)sha512ctx, res_sha512, (unsigned char*)NULL, 0); 
 
 	if (digs && digs->do_size) {
@@ -440,7 +440,7 @@ swlib_i_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int adjunc
 		strob_close(tmp);
 	}
 
-	if (digs && digs->do_sha1) {
+	if (digs && digs->do_sha1 == DIGS_ENABLE_ON) {
 		p = digs->sha1;
 		for (i = 0; i < (digest_hex_bytes / 2); ++i) {
 			sprintf(p,"%02x", res_sha1[i]);
@@ -450,7 +450,7 @@ swlib_i_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int adjunc
 		digs->sha1[40] = '\0';
 	}
 
-	if (digs && digs->do_sha512) {
+	if (digs && digs->do_sha512 == DIGS_ENABLE_ON) {
 		p = digs->sha512;
 		for (i = 0; i < (sha512digest_hex_bytes / 2); ++i) {
 			sprintf(p,"%02x", res_sha512[i]);
