@@ -19,7 +19,7 @@
 #include "swcommon_options.h"
 
 
-void 
+int
 main(int argc, char **argv)
 {
 	Header h;
