@@ -6,7 +6,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef strob_19990124_h
@@ -41,6 +41,7 @@ typedef struct {
 	int length_;	/* current length, not including the last NULL */
 	int extra_;	/* the lookahead allocation amount */
 	int reserve_;	/* the total length of reserved memory */
+	char in_use_;
 } STROB;
 
 
@@ -89,6 +90,8 @@ int 		strob_get_char	(STROB * strb, int index);
 char * 		strob_strcpy_at_offset(STROB * strb, int offset, char *str);
 
 void * 		strob_memcpy(STROB *strb, void * ct, size_t n);
+void *		strob_memcpy_at(STROB *strb, void * ct, size_t n, size_t offset);
+void		strob_append_hidden_null(STROB *strb);
 void *		strob_memmove(STROB *strb, void * ct, size_t n);
 void *		strob_memmove_to(STROB *strb, size_t dst_offset,
 						void * ct, size_t n);
