@@ -349,7 +349,7 @@ SWI * swi_create(void)
 	swi->nullfdM = open("/dev/null", O_RDWR);
 	memset(swi->tarbufM, '\0', sizeof(swi->tarbufM));
 	swi->distdataM = NULL;
-	swi->sh_dash_sM = (char*)NULL;
+	/* swi->sh_dash_sM = (char*)NULL; */
 	swi->opt_alt_catalog_rootM = 0;
 	swi->swicolM = swicol_create();
 	swi->exported_catalog_prefixM = NULL;
