@@ -145,7 +145,7 @@ typedef struct {	/* The Global package control structure */
 	int verboseM;		/* Controls debugging messages 		*/
 	char tarbufM[1024];
 	SWI_DISTDATA * distdataM;
-	char * sh_dash_sM;	/* the shell command			*/
+	/* char * sh_dash_sM; */ /* the shell command			*/
 	int opt_alt_catalog_rootM; /* -r option				*/
 	char * exported_catalog_prefixM; /* i.e.  <path>/catalog/ from  */
 				/* the distribution 			*/
