@@ -5,7 +5,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef swi_base_h_200501
 #define swi_base_h_200501
@@ -50,7 +50,7 @@ typedef struct {
 	char * tagM;
 	time_t create_timeM;
 	time_t mod_timeM;
-	int header_indexM;		/* index in the global INDEX file       */
+	int header_indexM;		/* index in the global INDEX file to relevant part */
 	SWHEADER * global_headerM;	/* Global INDEX access object	*/
 	SWVERID * swveridM;
 	int verboseM;			/* verbose level */
