@@ -5,7 +5,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,10 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
+#include "swuser_config.h"
 #include "ahs.h"
 #include "strob.h"
 #include "swlib.h"
@@ -416,9 +417,9 @@ void ahs_set_gid(AHS * xhs, gid_t gid) {
         ahs_vfile_hdr(xhs)->c_gid = gid;
 }
   
-void ahs_set_filesize(AHS * xhs, int filesize) { 
+void ahs_set_filesize(AHS * xhs, intmax_t filesize) { 
 	AHS_E_DEBUG("");
-        ahs_vfile_hdr(xhs)->c_filesize = filesize;
+        ahs_vfile_hdr(xhs)->c_filesize = (uintmax_t)filesize;
 }    
 
 void ahs_set_nlink(AHS * xhs, int  nlink) { 
@@ -611,9 +612,9 @@ ahs_get_tar_groupname(AHS * xhs)
 	return ahsStaticGetTarGroupname(xhs->file_hdrM);
 }
 
-size_t ahs_get_filesize(AHS * xhs) {
+intmax_t ahs_get_filesize(AHS * xhs) {
 	AHS_E_DEBUG("");
-	return (size_t)(ahs_vfile_hdr(xhs)->c_filesize);
+	return taru_hdr_get_filesize(ahs_vfile_hdr(xhs));
 }
 
 time_t ahs_get_mtime(AHS * xhs) {
