@@ -5,7 +5,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,10 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
+#include "swuser_config.h"
 #include "ahs.h"
 #include "strob.h"
 #include "swlib.h"
@@ -81,8 +82,11 @@ int
 etar_emit_header(ETAR * etar, int fd)
 {
 	int ret;
-	ret = write(fd, (void*)(HDR(etar)), TARRECORDSIZE);
-	if (ret != TARRECORDSIZE) return -1;
+	ret = atomicio((ssize_t (*)(int, void *, size_t))(write),
+		fd, (void*)(HDR(etar)), TARRECORDSIZE);
+	if (ret != TARRECORDSIZE) {
+		return -1;
+	}
 	return TARRECORDSIZE;
 }
 
@@ -93,11 +97,19 @@ etar_emit_data_from_fd(ETAR * etar, int ofd, int ifd)
 	int ret1;
 	int remains;
 	ret = swlib_pipe_pump(ofd, ifd);
-	if (ret < 0) return -1;
+	if (ret < 0) {
+		fprintf(stderr, "%s: etar_emit_data_from_fd(): loc=1: ret = %d\n",
+			swlib_utilname_get(), ret);
+		return -1;
+	}
 	if (ret) {
 		remains = TARRECORDSIZE - (ret % TARRECORDSIZE);
 		ret1 = swlib_pad_amount(ofd, remains);
-		if (ret1 < 0) return -2;
+		if (ret1 < 0) {
+			fprintf(stderr, "%s: etar_emit_data_from_fd(): loc=2: ret = %d\n",
+				swlib_utilname_get(), ret);
+			return -2;
+		}
 	} else {
 		remains = 0;
 	}
@@ -146,7 +158,7 @@ etar_set_size_from_fd(ETAR * etar, int fd, int * newfd)
 	int size;
 	int vfd;
 
-	*newfd = -1;
+	if (newfd) *newfd = -1;
 	if (uxfio_espipe(fd)) {
 		/*
 		 * can't seek
@@ -154,7 +166,9 @@ etar_set_size_from_fd(ETAR * etar, int fd, int * newfd)
 		tmp_fd = swlib_open_memfd();
 		swlib_pipe_pump(tmp_fd, fd);
 		vfd = tmp_fd;
-		*newfd = tmp_fd;
+		if (newfd == NULL)
+			fprintf(stderr, "usage error in etar_set_size_from_fd\n");
+		if (newfd) *newfd = tmp_fd;
 	} else {
 		vfd = fd;
 	}
@@ -279,7 +293,11 @@ etar_write_trailer_blocks(ETAR * etar, int ofd, int nblocks)
 	while (count-- > 0) {
 		ret = atomicio((ssize_t (*)(int, void *, size_t))(write),
 			ofd, (void*)(nullblock), (size_t)(512));
-		if (ret < 0) return -1;
+		if (ret < 0) {
+			fprintf(stderr, "%s: etar_write_trailer_blocks(): %s\n",
+				swlib_utilname_get(), strerror(errno));
+			return -1;
+		}
 		retval += 512;
 	}
 
