@@ -12,7 +12,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -22,7 +22,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 
@@ -59,21 +59,21 @@ void taru_swab_array (char * ptr, int count);
 /* Return 16-bit integer I with the bytes swapped.  */
 #define swab_short(i) ((((i) << 8) & 0xff00) | (((i) >> 8) & 0x00ff))
 
-int
+intmax_t
 taru_pump_amount2(int discharge_fd, int suction_fd,
-					int amount, int adjunct_ofd)
+			intmax_t amount, int adjunct_ofd)
 {
-	int ret;
-	int i = amount;
+	intmax_t ret;
+	intmax_t i = amount;
 	if ((ret=swlib_i_pipe_pump(suction_fd, discharge_fd,
-				&i, adjunct_ofd, taru_tape_buffered_read))) {
+				&i, adjunct_ofd, NULL, taru_tape_buffered_read))) {
 		return -1;
 	}
 	return i;
 }
 
-int
-taru_read_amount(int suction_fd, int amount)
+intmax_t
+taru_read_amount(int suction_fd, intmax_t amount)
 {
 	return taru_pump_amount2(-1, suction_fd, amount, -1);
 }
@@ -87,7 +87,7 @@ taru_tape_buffered_read(int fd, void * buf, size_t pcount)
 	TARU_E_DEBUG("ENTERING");
 	
 	taruib_fd = taruib_get_fd();
-	count = uxfio_sfread(fd, buf, pcount);
+	count = uxfio_sfa_read(fd, buf, pcount);
 
 	TARU_E_DEBUG2("file stats on descriptor %d", fd);
 	TARU_E_DEBUG2("file stats on descriptor %s", uxfio_dump_string(fd));
@@ -137,9 +137,8 @@ taru_read_header (TARU * taru, struct new_cpio_header * file_hdr,
 		if ((file_hdr->c_mode & CP_IFMT) == CP_IFLNK) {
 			link_name = (char *)malloc((unsigned int)
 						file_hdr->c_filesize + 1);
-			link_name[file_hdr->c_filesize] = '\0';
-			ret = taru_tape_buffered_read(in_des, link_name,
-							file_hdr->c_filesize);
+			link_name[(int)(file_hdr->c_filesize)] = '\0';
+			ret = taru_tape_buffered_read(in_des, link_name, (size_t)(file_hdr->c_filesize));
 			if (ret < 0) return -retval;
 			retval += ret;
 			ret = taru_tape_skip_padding(in_des,
@@ -177,7 +176,7 @@ taru_read_in_header (TARU * taru, struct new_cpio_header * file_hdr,
     {
         TARU_E_DEBUG("");
 	if ((retval=taru_read_in_tar_header2(taru, file_hdr, in_des,
-					(char*)(NULL), p_eoa, flags)) < 0) {
+					(char*)(NULL), p_eoa, flags, TARRECORDSIZE)) < 0) {
 		TARU_E_DEBUG("LEAVING");
       		return -1;
 	} else {
@@ -188,7 +187,7 @@ taru_read_in_header (TARU * taru, struct new_cpio_header * file_hdr,
 
   ahsStaticSetTarLinkname(file_hdr, NULL);
 
-  if ((retval=taru_tape_buffered_read(in_des, (void*)file_hdr, 6L)) != 6){
+  if ((retval=taru_tape_buffered_read(in_des, (void*)file_hdr, (size_t)(6))) != 6){
         TARU_E_DEBUG("");
     	fprintf (stderr, "error reading magic. retval= %d\n", retval);
 	TARU_E_DEBUG("LEAVING");
@@ -280,8 +279,7 @@ taru_read_in_header (TARU * taru, struct new_cpio_header * file_hdr,
 	}
       bytes_skipped++;
       memmove ((char *) file_hdr, (char *) file_hdr + 1, 5); 
-      if ((ret = taru_tape_buffered_read(in_des, 
-				(void*) ((char*)file_hdr + 5), 1L)) <= 0) {
+      if ((ret = taru_tape_buffered_read(in_des, (void*) ((char*)file_hdr + 5), (size_t)(1))) <= 0) {
 	    fprintf  (stderr, 
 		"error: header magic not found and subsequent read error.\n");
 	    TARU_E_DEBUG("LEAVING");
