@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,44 +17,50 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #ifndef uxfio_i_20020419_h
 #define uxfio_i_20020419_h
 
+#include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include "atomicio.h"
 
 #define UXFIO_I_NULL_FD -10001
 
+#define Xint intmax_t
+
 typedef struct {
 	int uxfdM;		/* unix file descriptor */
 	int buffertypeM;	/* memmory segment, disk file, dynamic (growable) memory */
 	int uxfd_can_seekM;	/* 1 if seekable file */
-	int posM;		/* pointer in buf , always >=start AND <end */
-	int startM;		/* beginning offset in buf of valid data, always 0 */
-	int endM;		/* pointer offset of first byte of invalid data */
-	int lenM;		/* length of buffer, memory length, or, maximum file length */
+	Xint posM;		/* pointer in buf , always >=start AND <end */
+	Xint startM;		/* beginning offset in buf of valid data, always 0 */
+	Xint endM;		/* pointer offset of first byte of invalid data */
+	Xint lenM;		/* length of buffer, memory length, or, maximum file length */
 	int errorM;		/* error */
-	int bytesreadM;		/* bytes read on virtual fd */
-	int current_offsetM;	/* curent offset in physical/logical file.*/
+	Xint bytesreadM;		/* bytes read on virtual fd */
+	Xint current_offsetM;	/* curent offset in physical/logical file.*/
+	Xint virtual_offsetM;	/* curent offset in physical/logical file.*/
 	int buffer_activeM;	/* 1 if any buffertype is in use */
 	char *bufM;		/* stream buffer for buffertype 0 */
 	int buffdM;		/* file buffer for buffertype 1 */
-	long offset_eof_savedM;	/* saved value of offset_eofM */
-	long offset_eofM;	/* virtual end-of marker for the uxfio_read function */
-	long offset_bofM;	/* virtual beginning of file. */
+	Xint offset_eof_savedM;	/* saved value of offset_eofM */
+	Xint offset_eofM;	/* virtual end-of marker for the uxfio_read function */
+	Xint offset_bofM;	/* virtual beginning of file. */
 	int auto_disableM;	/* if a subsequent read() will read from uxfd entirely, set buffer_active to 0 */
 	int auto_arm_delayM;	/* only disable buffer if auto_disable is set AND have previosly read from buffer */
 	int write_insertM;	/* true if inserting into a file. see UXFIO_F_WRITE_INSERT */
 	mode_t uxfio_modeM;
-	int v_endM;		/* length of file */
+	Xint v_endM;		/* length of file */
 	int use_countM;
 	char * tmpfile_rootdirM;
 	char buffilenameM[128];
@@ -69,9 +75,9 @@ typedef struct {
 	off_t (*vir_lseekM)(int  filedes, off_t offset, int whence);
 	int uxfio_fildesM;	/* Uxfio descriptor of this structure */
 	int lock_buf_fatalM;     /* If realloc shifts the address of the file image, then exit */
-	int output_block_sizeM;		/* 0 for no blocking */
-	int output_buffer_cM;		/* output buffer data length */
-	char * output_bufferM;		/* output buffering blocksize */
+	int output_block_sizeM;	
+	Xint output_buffer_cM;		/* output buffer data length */
+	char * output_bufferM;
 } UXFIO;
 
 UXFIO * uxfio_debug_get_object_address(int fd);
