@@ -1,7 +1,3 @@
-#ifdef DOTO_ID
-static char testswdefinitionwrite_cxx[] =
-"$Id$";
-#endif
 
 #include "swuser_config.h"
 #include <stdio.h>
